/**
 * 3dsLoad.c
 * 
 * Implements 3DS load facilities.
 *
 * @authors $Author: $ - Malf 
 * @version $Id: $
 */

#include "3dsLoad.h"      

static const char* filename= "dv.3ds";
static Lib3dsFile *file=0;
static int gl_width;
static int gl_height;
static GLuint dvDL;


/**
 *		getFile
 *		
 */
Lib3dsFile *getFile (void )
{
	return file;
}

/*
 *		load3DS
 *
 */
 void load3DS(const char* model) 
{
	filename = model;

	file=lib3ds_file_load(filename);

	if (file == NULL)
		printf("Arquivo %s nao encontrado!\n", model);
		
	lib3ds_file_eval(file,1.0);
}

/**
 *		render_node
 *
 */
static void render_node(Lib3dsNode *node)
{            
	int i;
	unsigned p;
	Lib3dsNode *pnode; 
	Lib3dsVector *normalL;
	Lib3dsObjectData *d;
	Lib3dsMatrix M;

	ASSERT(file);

    for (pnode=node->childs; pnode!=0; pnode=pnode->next) 
      render_node(pnode);
    
  
	if (node->type==LIB3DS_OBJECT_NODE) 
	{
		if (strcmp(node->name,"$$$DUMMY")==0) 
			return;
    
		if (!node->user.d) 
		{
			Lib3dsMesh *mesh=lib3ds_file_mesh_by_name(file, node->name);
			ASSERT(mesh);
			if (!mesh) 
				return;
		
			normalL=malloc(3*sizeof(Lib3dsVector)*mesh->faces);

			glPushMatrix();
			d=&node->data.object;
			glMultMatrixf(&node->matrix[0][0]);
			glTranslatef(-d->pivot[0], -d->pivot[1], -d->pivot[2]);

			lib3ds_matrix_copy(M, mesh->matrix);
			lib3ds_matrix_inv(M);
			glMultMatrixf(&M[0][0]);
        
			lib3ds_mesh_calculate_normals(mesh, normalL);

			for (p=0; p<mesh->faces; ++p) 
			{
				Lib3dsFace *f=&mesh->faceL[p];
				Lib3dsMaterial *mat=0;

				if (f->material[0]) 
					mat=lib3ds_file_material_by_name(file, f->material);
			
				if (mat) 
				{
					static GLfloat a[4]={0,0,0,1};
					float s;
					glMaterialfv(GL_FRONT, GL_AMBIENT, a);
					glMaterialfv(GL_FRONT, GL_DIFFUSE, mat->diffuse);
					glMaterialfv(GL_FRONT, GL_SPECULAR, mat->specular);
					s = pow(2, 10.0*mat->shininess);

					if (s>128.0) 
						s=128.0;

					glMaterialf(GL_FRONT, GL_SHININESS, s);
				} 
				else 
				{
					Lib3dsRgba a={0.2, 0.2, 0.2, 1.0};
					Lib3dsRgba d={0.8, 0.8, 0.8, 1.0};
					Lib3dsRgba s={0.0, 0.0, 0.0, 1.0};
					glMaterialfv(GL_FRONT, GL_AMBIENT, a);
					glMaterialfv(GL_FRONT, GL_DIFFUSE, d);
					glMaterialfv(GL_FRONT, GL_SPECULAR, s);
				}
          
				glBegin(GL_TRIANGLES);
				glNormal3fv(f->normal);

				for (i=0; i<3; ++i) 
				{
					glNormal3fv(normalL[3*p+i]);
					glVertex3fv(mesh->pointL[f->points[i]].pos);
				}
				glEnd();
			}
			
			free(normalL);
		}
		glPopMatrix();
	}
}

/**
 *		draw3dsNodes
 *
 */
 void draw3dsNodes(void)
 {
	 Lib3dsNode *p;

	for (p=file->nodes; p!=0; p=p->next) 
		render_node(p);
 }