/**
 * ARProcess.c
 * 
 * Image Processing and Recognition Module for the AR application.    
 *
 * @authors $Author: $ - Malf, Tucano, Sectoid
 * @version $Id: $
 */

#include "ARProcess.h"

int nModelIntersections;   
static int nModelLines;
float homografia[9];
static nLinesInLeftImage=0;
static nLinesInRightImage=0;
static jvLine modelLines[MAX_DETECTABLE_LINES];
static jvLine transformedModelLines[MAX_DETECTABLE_LINES];
static jvLine linesInLeftImage[MAX_DETECTABLE_LINES];
static jvLine linesInLeftImageSorted[MAX_DETECTABLE_LINES];
static jvLine linesInRightImage[MAX_DETECTABLE_LINES];
static jvPoint2D imageIntersections[MAX_INTERSECTIONS];
static jvPoint2D modelIntersections[MAX_INTERSECTIONS];
static unsigned char responseLeft[MAX_DETECTABLE_LINES];
static int intersectionMatrix[MAX_DETECTABLE_LINES][MAX_DETECTABLE_LINES];
static unsigned char *tmpBufferA = NULL;
static unsigned char *tmpBufferB = NULL;

//FLAGS
static int adjusting=0;
static int justlines=1;   //Flag para desenhar no flag 1 apenas linhas, nao exibe intersecoes
static int interpret=0;   //Flag para diferenciar as cores dos vertices das intersecoes das retas  interpretadas 
static int tsai=0;
static int draw=0;

//Tsai
static int np;
static int tsaiPoints=0;
static double WX[100], WY[100], FX[100], FY[100];
static double xw[100], yw[100], xf[100], yf[100];  //Abrir de acordo com o modelo! Mudar esse 10
static double xfn[100], yfn[100], xwn[100], ywn[100];
static double mtsaiopengl_modelview[16];
static double mtsaiopengl_projection[16];
static double focal, matTrans[3], matRot[9];
static double Tb[3], rb[9];
static double cam_X, cam_Y, cam_Z;

//cena
static float matrix[16] = { 1.0f,0.0f,0.0f,0.0f,
											0.0f,1.0f,0.0f,0.0f,
											0.0f,0.0f,1.0f,0.0f,
											0.0f,0.0f,0.0f,1.0f
										   };

static float current_frame=0.0;
Lib3dsFile *file=0;


/**
 *		drawObject
 *
 */
void drawObject (void)
{
	int viewport[4];
	int li=GL_LIGHT0;
    float lpos[4] = {0.0f,10.0f,10.0f,1.0f};
    GLfloat a[ ] = {0.0f, 0.0f, 0.0f, 1.0f};
    GLfloat c[ ] = {1.0f, 1.0f, 1.0f, 1.0f};
    GLfloat p[ ] = {0.0f, 0.0f, 0.0f, 1.0f};
    GLfloat camPos[ ] = {0.0f, 0.0f, 0.0f, 1.0f};
    GLfloat tarPos[ ] = {0.0f, 0.0f, 0.0f, 1.0f};
    Lib3dsLight *l;	
	Lib3dsMatrix M;

	glLightfv(GL_LIGHT0,GL_POSITION,lpos);
	glGetIntegerv(GL_VIEWPORT, viewport);
	glMatrixMode (GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity ();
	glLoadMatrixd (mtsaiopengl_projection);
	glMatrixMode (GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity ();
	glLoadMatrixd (mtsaiopengl_modelview);

	glClearColor(0.5, 0.5, 0.5, 1.0);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glDisable(GL_LIGHT1);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);

    glRotatef(90, 0,0,1);
	glTranslatef( 80, -100,0  ) ;
	glRotatef(180, 0,0,1);
	file = getFile();

    for (l=file->lights; l; l=l->next) 
	{
		glEnable(li);

		glLightfv(li, GL_AMBIENT, a);
		glLightfv(li, GL_DIFFUSE, c);
		glLightfv(li, GL_SPECULAR, c);

		p[0] = l->position[0];
		p[1] = l->position[1];
		p[2] = l->position[2];
		glLightfv(li, GL_POSITION, p);

		if (!l->spot_light) 
			continue;

		p[0] = l->spot[0] - l->position[0];
		p[1] = l->spot[1] - l->position[1];
		p[2] = l->spot[2] - l->position[2];      
		glLightfv(li, GL_SPOT_DIRECTION, p);
		++li;
    }

	lib3ds_matrix_camera(M, camPos, tarPos, 0.0f);
	glMultMatrixf(&M[0][0]);	
 
    draw3dsNodes ();

    current_frame+=1.0;

    if (current_frame>file->frames) 
		current_frame=0;
    
    lib3ds_file_eval(file, current_frame);
   
	glDisable(GL_LIGHTING);
	glDisable(GL_LIGHT0);
	glDisable(GL_CULL_FACE);
	glMatrixMode (GL_MODELVIEW);
	glPopMatrix();
	glMatrixMode (GL_PROJECTION);
	glPopMatrix();
 }

/**
 *		getIntersect
 *
 */
int getIntersect(float pa1x,float pa1y, float pa2x,float pa2y,
					      float pb1x,float pb1y, float pb2x,float pb2y, float *x,float *y)
{
	double dxa,dya,dxb,dyb,ca,cb,da,db;

	dxa = (pa1x-pa2x);
	dya = (pa1y-pa2y);
	dxb = (pb1x-pb2x);
	dyb = (pb1y-pb2y);

	if(dxa!=0 && dxb!=0)
	{
		ca = dya/dxa;
		cb = dyb/dxb;

		if(fabs(ca-cb)<0.1)
			return 0;

		da=(pa1y*pa2x - pa1x*pa2y)/dxa;
		db=(pb1y*pb2x - pb1x*pb2y)/dxb;
		*x=(db-da)/(cb-ca);
		*y=(da*cb-ca*db)/(ca-cb);

		return 1;
	}

	if(dxa==0 && dxb!=0)
	{
		cb=dyb/dxb;
		db=(pb1x*pb2y - pb1y*pb2x)/dxb;
		*x=pa1x;
		*y=pa1x*cb+db;
		return 1;
	}

	if(dxa!=0 && dxb==0)
	{
		ca=dya/dxa;
		da=(pa1x*pa2y - pa1y*pa2x)/dxa;
		*x=pb1x;
		*y=pb1x*ca+da;
		return 1;
	}
	return 0;
}

/**
 *		calculateIntersectionsInModel
 *
 */
void getIntersectionsInModel( void )
{
	float x,y;
	int i,j;
	nModelIntersections=0;

	for(i=0;i<nModelLines-1;i++)
		for(j=i+1;j<nModelLines;j++)
			intersectionMatrix[i][j]=0;

	for(i=0;i<nModelLines-1;i++)
		for(j=i+1;j<nModelLines;j++)
			if(getIntersect(modelLines[i].v1x,modelLines[i].v1y,modelLines[i].v2x,modelLines[i].v2y,
									modelLines[j].v1x,modelLines[j].v1y,modelLines[j].v2x,modelLines[j].v2y, &x, &y))
			{
				modelIntersections[nModelIntersections].x = x;
				modelIntersections[nModelIntersections].y = y;
				
				//tsai
				xw[nModelIntersections] = x;
				yw[nModelIntersections] = y;

				intersectionMatrix[i][j]=1;
				nModelIntersections++;
			}
}

/**
 *		getIntersectionsInImage
 *
 */
int getIntersectionsInImage(int nLines,jvLine *lines,jvPoint2D *point)
{
	float x,y;
	int i,j,k=0;

	tsaiPoints = 0;

	for(i=0;i<nLines-1;i++)
	{
		for(j=i+1;j<nLines;j++)
			if(intersectionMatrix[i][j]==1)
			{
				getIntersect(lines[i].v1x,lines[i].v1y,lines[i].v2x,lines[i].v2y,lines[j].v1x,
									lines[j].v1y,lines[j].v2x,lines[j].v2y,&x,&y);

				point[k].x=x;
				point[k].y=y;
				k++;

				//Tsai
				xf[tsaiPoints] = x;
				yf[tsaiPoints] = y;
				tsaiPoints++;
			}
	}
	
	return k;
}


/**
 *    getHomografiaPoint
 *
 *		A Homografia  encontrada atravs da equao abaixo:
 * 
 *			| x |		| h0  h1  h2 | | x |		| us |
 * 	H	| y | =	| h3  h4  h5 | | y |  =	| vs |
 *			| z |		| h6  h7  h8 | | w|		| ts	|
 *
 *		Com posse dos coeficientes H e dos pontos do modelo  possvel
 * encontrar a representao dos tais pontos na imagem capturada pela
 * camera. 
 *
 */
void getHomografiaPoint(float x, float y, float *xOut, float *yOut)
{
	float u, v, t,w;

	w = 1.0f;

   u = (homografia[0] * x) + (homografia[1] * y) + (homografia[2] * w);
   v = (homografia[3] * x) + (homografia[4] * y) + (homografia[5] * w);
    t = (homografia[6] * x) + (homografia[7] * y) + (homografia[8] * w);

	*xOut = u / t;
	*yOut = v / t;
}

/**
 *		getHomografiaLines
 *
 */
void getHomografiaLines( void )
{
	int n;
	for(n=0;n<nModelLines;n++)
	{
		getHomografiaPoint(modelLines[n].v1x,modelLines[n].v1y,&(transformedModelLines[n].v1x),&(transformedModelLines[n].v1y));
		getHomografiaPoint(modelLines[n].v2x,modelLines[n].v2y,&(transformedModelLines[n].v2x),&(transformedModelLines[n].v2y));
	}
}

/**
 *		getModelLines
 *
 */
void getModelLines (char *model_file_name)
{
	FILE *file;
	int n;
	if((file=fopen(model_file_name,"r"))==0)
	{
		printf("Error reading model file.\n");
		return;
	}

	fscanf(file,"%d",&nModelLines);

	for(n=0;n<nModelLines;n++)
		fscanf(file,"%f %f %f %f",&(modelLines[n].v1x),&(modelLines[n].v1y),&(modelLines[n].v2x),&(modelLines[n].v2y));

}

/**
 *		userRestriction
 *
 */
int userRestriction (int nlines_img, jvLine* lines_img, int nlines_mdl, jvLine* lines_mdl, unsigned char* current_interpretation)
{
	return 1;
}

/**
 *		userOpen
 *
 */
void userOpen(int width, int height, char *model_file_name)
{
	if (tmpBufferA==NULL) 
		tmpBufferA=(unsigned char *) malloc(width*height);

	if (tmpBufferB==NULL) 
		tmpBufferB=(unsigned char *) malloc(width*height);

	getModelLines(model_file_name);
	getIntersectionsInModel();
	jvInit(width,height,10,10,model_file_name);
}

/**
 *		userClose
 *
 */
void userClose( )
{
	if (tmpBufferA) 
		free(tmpBufferA);

	if (tmpBufferB) 
		free(tmpBufferB);

	jvClose( );
}

/**
 *		userProcessLeft
 *
 */
void userProcessLeft(int width, int height, unsigned char* inFrame, int* flags, float* values, unsigned char* outFrame)
{
	int x,y,k,n, m;	
	float w_homog ;
	static int contAjustedFrames=0;
	static int flag1=0;
	unsigned char *outBuffer=tmpBufferA;

	//limpando imagem apos descelecionar todos os botoes F
	if((flags[0]==0) && (flags[1]==0) && (flags[2]==0) && 
		(flags[3]==0) &&(flags[4]==0) &&(flags[5]==0))
	{
		justlines  = 1;
		adjusting = 0;
		interpret  = 0;
		tsai = 0;
		draw = 0;

		for(n=0;n<MAX_DETECTABLE_LINES;n++)
			responseLeft[n]=0;
	}

   k=0;
   nLinesInLeftImage=0;

   //1) Transformando a imagem capturada em monocromtica invertida
   for(y=0;y<height;y++){
		for(x=0;x<width;x++) {
			//tmpBufferA[y*width+x]= (unsigned char) (0.2126f*inFrame[k]+0.7152f*inFrame[k+1]+0.0722f*inFrame[k+2]);
			tmpBufferA[y*width+x]= (unsigned char)(255-(  0.2126f*inFrame[k] + 0.7152f*inFrame[k+1] + 0.0722f*inFrame[k+2] ));
			k+=3;
		}
   }

	//Se F0 for pressionado
   //2) Realiza o LoG e extrai os segmentos de retas longos
   if (flags[0]) 
   {
	    //Filtragem para o Realce de Linhas - LoG com Threshold
		jvLoGFilter(tmpBufferA,(int)80*values[0],tmpBufferB);

		//Extrao de segmentos de retas longos
		nLinesInLeftImage=jvEdgeDetect(tmpBufferB,20,linesInLeftImage, 1.0f);

		//Desenha as linhas encontradas no EdgeDetect, sem calcular as intersecoes
		justlines=1;

		//Se FI for pressionado
		//3) Calcula as intersecoes desses segmentos de reta encontrados
		//e desenha esses vrtices
		if (flags[1])
		{
			justlines=0;
			interpret=0;
			tsai = 0;
			draw = 0;
		}

		//Se FII for pressionado
		//4) Utiliza o interpretador de linhas para identificar as linhas vlidas capturadas 
		//e desenha esses vrtices
		if(flags[2])
		{		
			justlines=0;
			interpret=1;
			tsai = 0;
			draw = 0;

			if((!flags[3]) && (!flags[4]))
			{
				adjusting=0;
				jvInterpret(nLinesInLeftImage, linesInLeftImage, 1.0f, 1.0f, userRestriction, responseLeft);
			}

			//Se FIII for pressionado
			//5) Ajuste de linhas, caso se perca interpreta novamente
			if(flags[3]) 
			{			
				draw = 0;
				//Caso se perca
				if(!adjusting)
				{
					//Procuara a correspondncia entre as linhas (s uma vez)
					jvInterpret(nLinesInLeftImage, linesInLeftImage, 1.0f, 1.0f, userRestriction, responseLeft);

					//Ordena as linhas detectadas de acordo com a ordem do modelo
					for(n=0;n<nModelLines;n++)			
						linesInLeftImageSorted[responseLeft[n]]=linesInLeftImage[n];

					adjusting=1;
					getIntersectionsInImage(nModelLines,linesInLeftImageSorted,imageIntersections);
					nModelIntersections=9;
					jvFindTransf(homografia,nModelIntersections,modelIntersections,imageIntersections);
				}

				//Somente ajustando linhas
				else
				{			
					jvAdjust(tmpBufferB,nModelLines,transformedModelLines,(int)(values[1]*10));
					getIntersectionsInImage(nModelLines,transformedModelLines,imageIntersections);
					nModelIntersections=9;
					jvFindTransf(homografia,nModelIntersections,modelIntersections,imageIntersections);

					for(n=0;n<nModelLines;n++)
						//Testa se perdeu ou no o rastreamento
						if(transformedModelLines[n].valid==0)
						{
							adjusting=0;
							break;
						}
				}

				//Se FIV for apertado
				//TSAI
				if(flags[4])
				{
						tsai	 = 1;
						draw = 0;
						
						for( m=0 ; m<tsaiPoints ; m++)	
						{
							WX[m] = modelIntersections[m].x;
							WY[m] = modelIntersections[m].y;

							FX[m]		 =  homografia[0]*WX[m] + homografia[1]*WY[m] +homografia[2] ;
							FY[m]		 =  homografia[3]*WX[m] + homografia[4]*WY[m] +homografia[5] ;
							w_homog = homografia[6]*WX[m]  + homografia[7]*WY[m] +homografia[8] ;

							FX[m] = FX[m] / w_homog ;
							FY[m] = FY[m] / w_homog ;
							FX[m] = imageIntersections[m].x   - ( (float)IMAGE_W/2.0f );
							FY[m] = imageIntersections[m].y   - ( (float)IMAGE_H/2.0f  );
						}

						tsaiBuildNewData(tsaiPoints, WX, WY, FX, FY, NPTSX, NPTSY, WX, WY, FX, FY);
						tsaiCalibration2D( NPTSX*NPTSY,  WX,  WY,  FX,  FY,  &focal,  matTrans,  matRot );
						tsaiCameraPosition( &cam_X,  &cam_Y, &cam_Z, matTrans, matRot );

					//Se FV for apertado
					//TSAI parte 2
					if(flags[5])
					{
						tsaiOGLProjectionMatrix( focal,  100,  1000,  IMAGE_W,  IMAGE_H,  mtsaiopengl_projection );
						tsaiOGLModelViewMatrix(matTrans, matRot, mtsaiopengl_modelview);
						draw = 1;
					}
				}
			}
		}
		outBuffer=tmpBufferB;
	}

   k=0;
   for(y=0;y<height;y++)
      for(x=0;x<width;x++) {
         outFrame[k]=outFrame[k+1]=outFrame[k+2]=outBuffer[y*width+x];
         k+=3;
      }
}


/**
 *		userProcessRight
 *
 */
void userProcessRight(int width, int height, unsigned char* inFrame, int* flags, float* values, unsigned char* outFrame)
{
   int x,y,k; 
   unsigned char *outBuffer=tmpBufferA;
   k=0;
   nLinesInRightImage=0;
   for(y=0;y<height;y++)
      for(x=0;x<width;x++) {
         tmpBufferA[y*width+x]= (unsigned char) (0.2126f*inFrame[k]+0.7152f*inFrame[k+1]+0.0722f*inFrame[k+2]);
       k+=3;
      }
   k=0;
   for(y=0;y<height;y++)
      for(x=0;x<width;x++) {
         outFrame[k]=outFrame[k+1]=outFrame[k+2]=outBuffer[y*width+x];
         k+=3;
      }
}

/**
 *		userDrawLeftTransformed
 *
 */
void userDrawLeftTransformed(int width, int height)
{
   int n;
	glColor3f(0.96f,0.48f,0.09f);
	getHomografiaLines( );

	if(!tsai)
	{
		//desenhando as linhas reconstruidas do modelo
		glBegin(GL_LINES);
		for(n=0;n<nModelLines;n++){
			glVertex2f(transformedModelLines[n].v1x,transformedModelLines[n].v1y);
			glVertex2f(transformedModelLines[n].v2x,transformedModelLines[n].v2y);
		}
		glEnd();
		glFlush();
	}
}

/**
 *		userDrawLeftUnTransformed
 *
 */
void userDrawLeftUnTransformed(int width, int height)
{
   int n,i,j;
	float x,y;
	jvLine *linesToProcess;
	int nLinesToProcess;


	if(!adjusting)
	{
		linesToProcess=linesInLeftImage;
		nLinesToProcess=nLinesInLeftImage;
		glColor3f(1.0,0.0,0.0);
	}
	else
	{
		linesToProcess=linesInLeftImageSorted;
		nLinesToProcess=nModelLines;
		glColor3f(1.0,0.0,1.0);
	}

	//desenhando os segmentos de reta se for valido e nao estiver rodando o jvAdjust
	//Verificando se  permitido desenhar linhas
	if(justlines == 1)
	{
		glBegin(GL_LINES);
		for(n=0;n<nLinesToProcess;n++)
		{
				if(!adjusting && responseLeft[n]==255)
					continue;

				glVertex2f(linesToProcess[n].v1x,linesToProcess[n].v1y);
				glVertex2f(linesToProcess[n].v2x,linesToProcess[n].v2y);
		}
		glEnd();
	}

	//zerando qtde de vertices			
	np=0; 

	for(i=0;i<nLinesToProcess-1;i++)
	{
		if(!adjusting && responseLeft[i]==255)
			continue;

		for(j=i+1;j<nLinesToProcess;j++)
		{
			if(!adjusting && responseLeft[j]==255)
				continue;
			
			//Verificando se  permitido desenhar intersecoes ou tsai
			//if((!justlines) && (!tsai))
			if(!justlines)
			{
				if(getIntersect(linesToProcess[i].v1x,linesToProcess[i].v1y,linesToProcess[i].v2x,linesToProcess[i].v2y,
										linesToProcess[j].v1x,linesToProcess[j].v1y,linesToProcess[j].v2x,linesToProcess[j].v2y,&x,&y))
				{
						if ((x>320) || (x<0) || (y>240)|| (y<0))
							continue;

						if(!tsai)
						{
							glPointSize(8);
							
							if(!interpret)	
								glColor3f(0.0,0.0,1.0);

							else
								glColor3f(0.2,0.8,0.2);

								glBegin(GL_POINTS);
								glVertex2f(x,y);
								glEnd();							
						}
				}
			}
		}
	}
	
	if(tsai && adjusting && !draw)
	{
		glColor3d(0.07, 0.8, 1.0);
		glPointSize(5.0);
		glBegin(GL_POINTS);

		for (i=0; i<NPTSX*NPTSY; i++)
			glVertex3d(FX[i] +   ( (float)IMAGE_W/2.0f ), FY[i] +  ( (float)IMAGE_H/2.0f ), 0.0);

		glEnd();
	}

   glFlush();
}

/**
 *		userDrawLeft
 *
 */
void userDrawLeft(int width, int height)
{
	//apenas reconstruir o modelo
	if((adjusting) && (!tsai))
		userDrawLeftTransformed(width, height);
	
	//apenas vertices
	else if ((!adjusting) && (!tsai))
		userDrawLeftUnTransformed(width, height);

	//tsai + ajuste
	else if ((adjusting) && (tsai))
	{
		getHomografiaLines();
		userDrawLeftUnTransformed(width, height);
	}

	glEnable (GL_DEPTH_TEST);
    glDepthFunc (GL_LEQUAL);
    glClear( GL_DEPTH_BUFFER_BIT );

	if(draw)
		drawObject( );

}

/**
 *		userDrawRight
 *
 */
void userDrawRight(int width, int height)
{
	int n;
	glColor3f(1.0,0.0,0.0);
	glBegin(GL_LINES);

	for(n=0;n<nLinesInRightImage;n++)
	{
		glVertex2f(linesInRightImage[n].v1x,linesInRightImage[n].v1y);
		glVertex2f(linesInRightImage[n].v2x,linesInRightImage[n].v2y);
	}
	glEnd();
	glFlush();
}