/**
 * Image.c
 * 
 *
 * @authors $Author: $ - Malf
 * @version $Id: $
 */


#include "image.h"


typedef unsigned long int LONG;   //***    

Image * imageT;

int xant,yant,xrec,yrec, xint, yint;


/************************************************************************/
/* Definicao das Funcoes Privadas                                       */
/************************************************************************/

/*  getuint e putuint:
 * Funcoes auxiliares para ler e escrever inteiros na ordem (lo-hi)
 * Note  que  no Windows as variaveis tipo "unsigned short int" sao 
 * armazenadas  no disco em dois bytes na ordem inversa. Ou seja, o 
 * numero  400, por exemplo, que  pode ser escrito como 0x190, fica 
 * armazenado  em dois bytes consecutivos 0x90 e 0x01. Nos sistemas 
 * UNIX e Mac este mesmo inteiro seria armazenado  na  ordem 0x01 e 
 * 0x90. O armazenamento do Windows e' chamado  de  "little endian"  
 * (i.e., lowest-order byte stored first), e  no  sitemas  Unix sao
 * "big-endian" (i.e., highest-order byte stored first). 
 */

/***************************************************************************
* Reads an unsigned integer from input                                     *
***************************************************************************/
static int getuint(unsigned short *uint, FILE *input)
{
  int got;
  unsigned char temp[2];
  unsigned short tempuint;

  got = (int) fread(&temp, 1, 2, input);
  if (got != 2) return 0;

  tempuint = ((unsigned short)(temp[1])<<8) | ((unsigned short)(temp[0]));

  *uint = tempuint;

  return 1;
}

/***************************************************************************
* Writes an unsigned integer in output                                     *
***************************************************************************/
static int putuint(unsigned short uint, FILE *output)
{
  int put;
  unsigned char temp[2];

  temp[0] = uint & 0xff;
  temp[1] = (uint >> 8) & 0xff;

  put = (int) fwrite(&temp, 1, 2, output);
  if (put != 2) return 0;
 
  return 1;
}

/***************************************************************************
* Reads a long integer from input                                          *
***************************************************************************/ 
static int getlong(FILE *input, long int *longint)
{
  int got;
  unsigned char temp[4];
  long int templongint;

  got = (int)fread(&temp, 1, 4, input);
  if (got != 4) return 0;

  templongint = ((long int)(temp[3])<<24) | ((long int)(temp[2])<<16) 
    | ((long int)(temp[1])<<8) | ((long int)(temp[0]));

  *longint = templongint;

  return 1;
}

/***************************************************************************
* Writes a long integer in output                                          *
***************************************************************************/ 
static int putlong(FILE *output, long int longint)
{
  int put;
  unsigned char temp[4];

  temp[0] = (unsigned char)longint & 0xff;
  temp[1] = (unsigned char)(longint >> 8) & 0xff;
  temp[2] = (unsigned char)(longint >> 16) & 0xff;
  temp[3] = (unsigned char)(longint >> 24) & 0xff;

  put = (int)fwrite(&temp, 1, 4, output);

  if (put != 4) return 0;

  return 1;
}

/***************************************************************************
* Reads a word from input                                                  *
***************************************************************************/ 
static int getword(FILE *input, unsigned short int *word)
{
  int got;
  unsigned char temp[2];
  unsigned short int tempword;

  got = (int)fread(&temp, 1, 2, input);
  if (got != 2) return 0;

  tempword = ((unsigned short int)(temp[1])<<8) | ((unsigned short int)(temp[0]));

  *word = tempword;

  return 1;
}

/***************************************************************************
* Writes a word in output                                                  *
***************************************************************************/ 
static int putword(FILE *output, unsigned short int word)
{
  int put;
  unsigned char temp[2];

  temp[0] = word & 0xff;
  temp[1] = (word >> 8) & 0xff;

  put = (int)fwrite(&temp, 1, 2, output);
  if (put != 2) return 0;

  return 1;
}

/***************************************************************************
* Reads a double word from input                                           *
***************************************************************************/ 
static int getdword(FILE *input, unsigned long int *dword)
{
  int got;
  unsigned char temp[4];
  unsigned long int tempdword;

  got = (int)fread(&temp, 1, 4, input);
  if (got != 4) return 0;

  tempdword = ((unsigned long int)(temp[3])<<24) | ((unsigned long int)(temp[2])<<16) 
    | ((unsigned long int)(temp[1])<<8) | ((unsigned long int)(temp[0]));

  *dword = tempdword;

  return 1;
}

/***************************************************************************
* Writes a double word in output                                           *
***************************************************************************/ 
static int putdword(FILE *output, unsigned long int dword)
{
  int put;
  unsigned char temp[4];

  temp[0] = (unsigned char) (dword & 0xff);
  temp[1] = (unsigned char) ((dword >>  8) & 0xff);
  temp[2] = (unsigned char) ((dword >> 16) & 0xff);
  temp[3] = (unsigned char) ((dword >> 24) & 0xff);

  put = (int)fwrite(&temp, 1, 4, output);

  if (put != 4) return 0;

  return 1;
}

/************************************************************************/
/* Definicao das Funcoes Exportadas                                     */
/************************************************************************/

Image * imgCreate(int w, int h)
{
   Image * image = (Image*) malloc (sizeof(Image));
   assert(image);
   image->width  =(unsigned int) w;
   image->height =(unsigned int) h;
   //***image->buf = (float *) malloc (w * h * 3*sizeof(float));
   image->buf = (unsigned char *) malloc (w * h * 3*sizeof(char));
   assert(image->buf);
   return image;
}



void imgDestroy(Image * image)
{
   if (image)
   {
      if (image->buf) free (image->buf);
      free(image);
   }
}



Image * imgGrey(Image * image)
{
	int w = imgGetWidth(image);
	int h = imgGetHeight(image);
	int x,y;
	float rgb[3],grey[3];

	for (y=0;y<h;y++){
		for (x=0;x<w;x++) {
			imgGetPixel3fv(image,x,y,rgb);
			grey[0]=0.2126f*rgb[0]+0.7152f*rgb[1]+0.0722f*rgb[2];
			grey[1]=grey[0];
			grey[2]=grey[0];
			imgSetPixel3fv(image,x,y,grey);
		}
	}

	return image;
}


/************************************************************************/
/* [ imgResize  ]                                     
/*
/* Resizes a image to a new resolution.
/* 
/************************************************************************/

Image * imgResize(Image * img0, int w1, int h1) 
{
   int x0,y0,x1,y1;
   float w0 = (float) img0->width; 
   float h0 = (float) img0->height;
	float color[3];

   Image * img1 = imgCreate(w1,h1);

   for (y1=0;y1<h1;y1++)
   {
		for (x1=0;x1<w1;x1++)
		{
         /* Getting the nearest color */
			x0=ROUND(w0*x1/w1);
			y0=ROUND(h0*y1/h1);
			imgGetPixel3fv(img0,x0,y0,color);
			imgSetPixel3fv(img1,x1,y1,color);
		}
   }
   
   return img1;
}


Image * imgAdjust2eN(Image *img0)
{
	Image * img1;
	int i=0,valid[14]={1,2,4,8,16,32,64,128,512,1024,2048,4096,8192,16384};
	int w0=img0->width;
	int h0=img0->height;
	int w1,h1;
	int x,y;
	float rgb[3],black[3]={0.f,0.f,0.f};

	for (i=0;i<14&&valid[i]<w0;i++);
	w1=valid[i];

	for (i=0;i<14&&valid[i]<h0;i++);
	h1=valid[i];

	img1=imgCreate(w1,h1);

	for (y=0;y<h1;y++){
		for (x=0;x<w1;x++) {
			if (x<w0&&y<h0) {
				imgGetPixel3fv(img0,x,y,rgb);
				imgSetPixel3fv(img1,x,y,rgb);
			}
			else
				imgSetPixel3fv(img1,x,y,black);
		}
	}

	return img1;
}

int imgGetWidth(Image * image)
{
	return image->width;
}

int imgGetHeight(Image * image)
{
	return image->height;
}

unsigned char * imgGetRGBData(Image * image)
{
    return image->buf;
}

//***void imgSetRGBData(Image * image, float* buffer)
void imgSetRGBData(Image * image, unsigned char* buffer)
{
	unsigned int i;

	for(i=0; i<3*image->height*image->width; i++)
		image->buf[i] = buffer[i];
}

void imgSetPixel3fv(Image * image, int x, int y, float * color)
{
   int pos = (y*image->width*3) + (x*3);

   image->buf[pos  ] = (unsigned char)(color[0] * 255.);
   image->buf[pos+1] = (unsigned char)(color[1] * 255.);
   image->buf[pos+2] = (unsigned char)(color[2] * 255.); //***
}

void imgGetPixel3fv(Image * image, int x, int y, float *color)
{
   int pos = (y*image->width*3) + (x*3);

   color[0] = (float)(image->buf[pos])   / 255.0f;
   color[1] = (float)(image->buf[pos+1]) / 255.0f;
   color[2] = (float)(image->buf[pos+2]) / 255.0f;
}

void imgSetPixel3ubv(Image * image, int x, int y, unsigned char * color)
{
   int pos = (y*image->width*3) + (x*3);

   image->buf[pos  ] = color[0];
   image->buf[pos+1] = color[1];
   image->buf[pos+2] = color[2]; //***
}

void imgGetPixel3ubv(Image * image, int x, int y, unsigned char *color)
{
   int pos = (y*image->width*3) + (x*3);
   int r= ROUND (255*image->buf[pos]);
   int g= ROUND (255*image->buf[pos+1]);
   int b= ROUND (255*image->buf[pos+2]);
   color[0] = (unsigned char)(r<256) ? r : 255 ;
   color[1] = (unsigned char)(g<256) ? g : 255 ;
   color[2] = (unsigned char)(b<256) ? b : 255 ;
}

Image * imgReadTGA(char *filename) 
{
   FILE        *filePtr;

   Image       *image;                /* imagem a ser criada */
   unsigned char *buffer;             /* buffer para ler o vetor de rgb da imagem  */

   unsigned char imageType;        /* 2 para imagens RGB */ 
   unsigned short int imageWidth;  /* largura da imagem */
   unsigned short int imageHeight; /* altura da imagem */
   unsigned char bitCount;         /* numero de bits por pixel */
 
   int    x,y;         /* posicao de um pixel  */
  
   unsigned char ucharSkip;      /* dado lixo unsigned char */
   short int     sintSkip;       /* dado lixo short int */

   /* abre o arquivo com a imagem TGA */
   filePtr = fopen(filename, "rb");
   assert(filePtr);

   /* pula os primeiros dois bytes que devem ter valor zero */
   ucharSkip = getc(filePtr); /* tamanho do descritor da imagem (0) */
   if (ucharSkip != 0) printf("erro na leitura de %s: imagem com descritor\n", filename);

   ucharSkip = getc(filePtr); 
   if (ucharSkip != 0) printf("erro na leitura de %s: imagem com tabela de cores\n", filename);
   
   /* le o tipo de imagem (que deve ser obrigatoriamente 2).  
      nao estamos tratando dos outros tipos */
   imageType=getc(filePtr);
   assert(imageType == 2);

   /* pula 9 bytes relacionados com a tabela de cores 
     (que nao existe quando a imagem e' RGB, imageType=2) */
   getuint((short unsigned int *)&sintSkip,filePtr);
   getuint((short unsigned int *)&sintSkip,filePtr);
   ucharSkip = getc(filePtr); 

   /* especificacao da imagem */
   getuint((short unsigned int *)&sintSkip,filePtr);      /* origem em x (por default = 0) */
   getuint((short unsigned int *)&sintSkip,filePtr);      /* origem em y (por default = 0) */ 
   getuint(&imageWidth,filePtr);    /* largura */
   getuint(&imageHeight,filePtr);   /* altura */

   /* read image bit depth */
   bitCount=getc(filePtr);
   assert(bitCount == 24);  /* colorMode -> 3 = BGR (24 bits) */

   /* read 1 byte of garbage data */
   ucharSkip = getc(filePtr); 

   /* cria uma instancia do tipo Imagem */
   image = imgCreate(imageWidth,imageHeight);
   buffer = (unsigned char *) malloc(3*imageWidth*imageHeight*sizeof(unsigned char));
   assert(image);
   assert(buffer);

   /* read in image data */
   fread(buffer, sizeof(unsigned char), 3*imageWidth*imageHeight, filePtr);
   
   /* copia e troca as compontes de BGR para RGB */
   /* change BGR to RGB so OpenGL can read the image data */
   for (y=0;y<imageHeight;y++) {
	   for (x=0;x<imageWidth;x++) {
           int pos = (y*imageWidth*3) + (x*3);
           //***image->buf[pos  ] = (float)(buffer[pos+2]/255.);
           //***image->buf[pos+1] = (float)(buffer[pos+1]/255.);
           //***image->buf[pos+2] = (float)(buffer[pos  ]/255.);
           image->buf[pos  ] = buffer[pos+2];
           image->buf[pos+1] = buffer[pos+1];
           image->buf[pos+2] = buffer[pos  ];
	   }
   }
   free(buffer);
   fclose(filePtr);
   return image;
}


/************************************************************************/
/* [ imgWriteTGA  ]                                     
/*
/* 
/* 
/************************************************************************/


int imgWriteTGA(char *filename, Image * image)
{
   unsigned char imageType=2;     /* RGB(A) sem compresso */
   unsigned char bitDepth=24;     /* 24 bits por pixel */

   FILE          *filePtr;        /* ponteiro do arquivo */
   unsigned char * buffer;        /* buffer de bytes */
   unsigned int  x,y;   

   unsigned char byteZero=0;      /* usado para escrever um byte zero no arquivo */
   short int     shortZero=0;     /* usado para escrever um short int zero no arquivo */

   if (!image) return 0;

   /* cria um arquivo binario novo */
   filePtr = fopen(filename, "wb");
   assert(filePtr);

   /* cria o buffer */
   buffer = (unsigned char *) malloc(3*image->width*image->height*sizeof(unsigned char));
   assert(buffer);

  /* copia e troca as compontes de BGR para RGB */
   for (y=0;y<image->height;y++) {
	   for (x=0;x<image->width;x++) {
           int pos = (y*image->width*3) + (x*3);
           //*** int r= ROUND (255*image->buf[pos  ]);
           //*** int g= ROUND (255*image->buf[pos+1]);
           //*** int b= ROUND (255*image->buf[pos+2]);
           int r= image->buf[pos  ];
           int g= image->buf[pos+1];
           int b= image->buf[pos+2];

           buffer[pos+2] = (unsigned char)(r<256) ? r : 255 ;
           buffer[pos+1] = (unsigned char)(g<256) ? g : 255 ;
           buffer[pos  ] = (unsigned char)(b<256) ? b : 255 ;
  	   }
   }

   /* escreve o cabecalho */
   putc(byteZero,filePtr);          /* 0, no. de caracteres no campo de id da imagem */
   putc(byteZero,filePtr);          /* = 0, imagem nao tem palheta de cores */
   putc(imageType,filePtr);         /* = 2 -> imagem "true color" (RGB) */
   putuint(shortZero,filePtr);      /* info sobre a tabela de cores (inexistente) */
   putuint(shortZero,filePtr);      /* idem */
   putc(byteZero,filePtr);          /* idem */
   putuint(shortZero,filePtr);      /* =0 origem em x */
   putuint(shortZero,filePtr);      /* =0 origem em y */
   putuint(image->width,filePtr);   /* largura da imagem em pixels */
   putuint(image->height,filePtr);  /* altura da imagem em pixels */
   putc(bitDepth,filePtr);          /* numero de bits de um pixel */
   putc(byteZero, filePtr);         /* =0 origem no canto inf esquedo sem entrelacamento */

   /* escreve o buf de cores da imagem */
   fwrite(buffer, sizeof(unsigned char), 3*image->width*image->height, filePtr);

   fprintf(stdout,"imgWriteTGA: %s successfuly generated\n",filename);
   free(buffer);
   fclose(filePtr);
   return 1;
}




Image * imgReadBMP(char *filename)
{
	FILE  *filePtr;				/* ponteiro do arquivo */
	Image *image;				/* imagem a ser criada */
	BYTE *linedata;
	
	USHORT  bfType;             /* "BM" = 19788           */
	LONG    biWidth;            /* image width in pixels  */
	LONG    biHeight;           /* image height in pixels */
	WORD    biBitCount;         /* bitmap color depth     */
	DWORD   bfSize;

	USHORT  ushortSkip;			/* dado lixo USHORT */
	DWORD   dwordSkip;			/* dado lixo DWORD  */
	LONG    longSkip;			/* dado lixo LONG   */
	WORD    wordSkip;			/* dado lixo WORD   */

	LONG i, j, k, l, linesize, got;

	/* abre o arquivo com a imagem BMP */
	filePtr = fopen(filename, "rb");
	assert(filePtr);

	/* verifica se eh uma imagem bmp */
	getuint(&bfType, filePtr);
	assert(bfType == 19778);

	/* pula os 12 bytes correspondentes a bfSize, Reserved1 e Reserved2 */
	getdword(filePtr, &bfSize);
	getuint(&ushortSkip, filePtr);  /* Reserved1, deve ter valor 0 */
	assert(ushortSkip == 0);
	getuint(&ushortSkip, filePtr);  /* Reserved2, deve ter valor 0 */
	assert(ushortSkip == 0);

	/* pula os 4 bytes correspondentes a bfOffBits, que deve ter valor 54 */
	getdword(filePtr, &dwordSkip);
	assert(dwordSkip == 54);
	
	/* pula os 4 bytes correspondentes a biSize, que deve ter valor 40 */
	getdword(filePtr, &dwordSkip);
	assert(dwordSkip == 40);

	/* pega largura e altura da imagem */
	getlong(filePtr, &biWidth);
	getlong(filePtr, &biHeight);
	
	/* verifica que o numero de quadros eh igual a 1 */
	getword(filePtr, &wordSkip);
	assert(wordSkip == 1);

	/* Verifica se a imagem eh de 24 bits */
	getword(filePtr, &biBitCount);
	if(biBitCount != 24)
	{
		fprintf(stderr, "imgReadBMP: Not a bitmap 24 bits file.\n");		
        fclose(filePtr);
        return (NULL);
	}

	/* pula os demais bytes do infoheader */
	getdword(filePtr, &dwordSkip);
	assert(dwordSkip == 0);
	getdword(filePtr, &dwordSkip);
	getlong(filePtr, &longSkip);
	getlong(filePtr, &longSkip);
	getdword(filePtr, &dwordSkip);
	getdword(filePtr, &dwordSkip);

	image = imgCreate(biWidth, biHeight);

	/* a linha deve terminar em uma fronteira de dword */
	linesize = 3*image->width;
	if (linesize & 3) {
		linesize |= 3;
		linesize++;
	}
  
	/* aloca espaco para a area de trabalho */ 
	linedata = (BYTE *) malloc(linesize);
	if (linedata == NULL) {
		fprintf(stderr, "get24bits: Not enough memory.\n");
		return 0;
	}
    
	/* pega as componentes de cada pixel */
	for (k=0, i=0; i<image->height; i++) {
		got = (unsigned long int)fread(linedata, linesize, 1, filePtr);
		if (got != 1) {
			free(linedata);
			fprintf(stderr, "get24bits: Unexpected end of file.\n");
		}
		for (l=1, j=0; j<image->width; j++, l=l+3) {
			//***image->buf[k++] = (float)(linedata[l+1]/255.);
			//***image->buf[k++] = (float)(linedata[l  ]/255.); 
			//***image->buf[k++] = (float)(linedata[l-1]/255.);
			image->buf[k++] = linedata[l+1];
			image->buf[k++] = linedata[l  ]; 
			image->buf[k++] = linedata[l-1];

		}
	}

	free(linedata);
	fclose(filePtr);
	return image;
}



/************************************************************************/
/* [ imgWriteBMP  ]                                     
/*
/* 
/* 
/************************************************************************/


int imgWriteBMP(char *filename, Image * bmp)
{
	FILE          *filePtr;		   /* ponteiro do arquivo */
	unsigned char *filedata;
	DWORD bfSize;
	unsigned int i, j, k, l;
   

	int linesize, put;

	if (!bmp) return 0;

	/* cria um novo arquivo binario */
	filePtr = fopen(filename, "wb");
	assert(filePtr);

	/* a linha deve terminar em uma double word boundary */
	linesize = bmp->width * 3;
	if (linesize & 3) {
		linesize |= 3;
		linesize ++;
	}

	/* calcula o tamanho do arquivo em bytes */
	bfSize = 14 +							/* file header size */
			   40 +							/* info header size */
			   bmp->height * linesize;	/* image data  size */

	/* Preenche o cabealho -> FileHeader e InfoHeader */
	putuint(19778, filePtr);		    	/* type = "BM" = 19788 */
	putdword(filePtr, bfSize);				/* bfSize -> file size in bytes */
	putuint(0, filePtr);						/* bfReserved1, must be zero */
	putuint(0, filePtr);						/* bfReserved2, must be zero */
	putdword(filePtr, 54);					/* bfOffBits -> offset in bits to data	*/

	putdword(filePtr, 40);					/* biSize -> structure size in bytes  */
	putlong(filePtr, bmp->width);			/* biWidth -> image width in pixels	 */			
	putlong(filePtr, bmp->height);		/* biHeight -> image height in pixels */
	putword(filePtr, 1);						/* biPlanes, must be 1 */
	putword(filePtr, 24);					/* biBitCount, 24 para 24 bits -> bitmap color depth */
	putdword(filePtr, 0);					/* biCompression, compression type -> no compression */
	putdword(filePtr, 0);					/* biSizeImage, nao eh usado sem compressao */
	putlong(filePtr, 0);						/* biXPelsPerMeter */
	putlong(filePtr, 0);						/* biYPelsPerMeter */
	putdword(filePtr, 0);					/* biClrUsed, numero de cores na palheta */
	putdword(filePtr, 0);					/* biClrImportant, 0 pq todas sao importantes */

	/* aloca espacco para a area de trabalho */
	filedata = (unsigned char *) malloc(linesize);
	assert(filedata);

	/* a linha deve ser zero padded */
	for (i=0; i<(linesize-(3*bmp->width)); i++) 
		filedata[linesize-1-i] = 0;
	j = 1;


	for (k=0; k<bmp->height;k++)
	{
		l = 1;
		// coloca as componentes BGR no buffer    
		for (i=0; i<bmp->width; i++) {
			//*** int b= ROUND (255*bmp->buf[j+1]);
			//*** int g= ROUND (255*bmp->buf[j  ]);
			//*** int r= ROUND (255*bmp->buf[j-1]);
			int b= bmp->buf[j+1];
			int g= bmp->buf[j  ];
			int r= bmp->buf[j-1];

			filedata[l-1] = (unsigned char)(b<256) ? b : 255 ;
			filedata[l  ] = (unsigned char)(g<256) ? g : 255 ;
			filedata[l+1] = (unsigned char)(r<256) ? r : 255 ;
			j+=3; l+=3;
		}

		//joga para o arquivo
		put = (int)fwrite(filedata, linesize, 1, filePtr);
		if (put != 1) {
		fprintf(stderr, "put24bits: Disk full.");
		free(filedata);
		return 0;
		}
	}


	/* operacao executada com sucesso */
	fprintf(stdout,"imgWriteBMP: %s successfuly generated\n",filename);
	free(filedata);
	fclose(filePtr);
	return 1;
}

int imgWriteLine(Image * image, char *filename)
{
   FILE         *filePtr;         /* ponteiro do arquivo */
   unsigned int x,y=image->height/3;

   /* cria um arquivo binario novo */
   filePtr = fopen(filename, "wt");
   assert(filePtr);

   for (x=0;x<image->width;x++)
   {
       int pos = (y*image->width*3) + (x*3);
	   fprintf(filePtr,"%f\t%f\t%f\n",image->buf[pos],image->buf[pos+1],image->buf[pos+2]);
//	   image->buf[pos  ]=0.f;
//	   image->buf[pos+1]=0.f;
//	   image->buf[pos+2]=0.f;
	   image->buf[pos  ] = 0;
	   image->buf[pos+1] = 0;
	   image->buf[pos+2] = 0;

   }
   fclose(filePtr);
   return 1;
}

int imgWriteHistogram(Image * image, char *filename)
{
   FILE         *filePtr;         /* ponteiro do arquivo */
   long int histR[256],histG[256],histB[256];

   int x,y,i;
   int w = imgGetWidth(image);
   int h = imgGetHeight(image);
   unsigned char rgb[3];

   /* cria um arquivo binario novo */
   filePtr = fopen(filename, "wt");
   assert(filePtr);

   for (i=0;i<256;i++) {
	   histR[i]=0;
	   histG[i]=0;
	   histB[i]=0;
   }

   for (y=0;y<h;y++) {
	   for (x=0;x<w;x++) 
	   {
		   imgGetPixel3ubv(image,x,y,rgb);
	       histR[rgb[0]]++;
	       histG[rgb[1]]++;
	       histB[rgb[2]]++;
	   }
   }

   for (i=0;i<256;i++) 
	   fprintf(filePtr,"%d,%d,%d\n",histR[i],histG[i],histB[i]);
   
   fclose(filePtr);
   return 1;
}

static int comparaCor(const void * p1, const void * p2)
{
	float *c1 = (float *) p1;  /* aponta para o byte red da cor 1 */
	float *c2 = (float *) p2;  /* aponta para o byte red da cor 2 */

	/* compara o canal vermelho */
	if (*c1 < *c2) return -1;
	if (*c1 > *c2) return  1;
	
	/* compara  o canal verde, uma vez que o vermelho e' igual */
	c1++; c2++;
    if (*c1 < *c2) return -1;
    if (*c1 > *c2) return  1;
	
	/* compara o canal azul, uma vez que o vermelho e o azul sao iguais */
    c1++; c2++;
    if (*c1 < *c2) return -1;
    if (*c1 > *c2) return  1;
    
	/* sao iguais */
	return 0;
}

/* --- Funcoes do primeiro trabalho ----*/

unsigned int imgCountColors(Image * img)
{
	unsigned int numCor = 1;
 	int w = imgGetWidth(img);
	int h = imgGetHeight(img);
	float* buf=(float *) imgGetRGBData(img);
	float *vet=(float*) malloc(3*w*h*sizeof(float));
	int i;

	/* copia o buffer da imagem no vetor de floats fazendo uma quantizacao para 256 tons de cada componente de cor */
	for (i=0;i<3*w*h;i++) {
		float val = (float)(255*buf[i]);
        vet[i]  = (float)(val<256) ? val : 255;
	}

	/* ordena o vetor */
	qsort(vet,w*h,3*sizeof(float),comparaCor);

	/* conta o numero de cores diferentes */
	for (i=3; i<3*w*h; i+=3)
		if (comparaCor(&vet[i-3],&vet[i])!=0) numCor++;

	free(vet);
	return numCor;
}

Image * imgNormalizeColors(Image * img0)
{
	int w = imgGetWidth(img0);
	int h = imgGetHeight(img0);
	Image *img1=imgCreate(w,h);
	int x,y;
	float rgb[3];


	for (y=0;y<h;y++){
		for (x=0;x<w;x++) {
			imgGetPixel3fv(img0,x,y,rgb);
			imgSetPixel3fv(img1,x,y,rgb);
		}
	}

	return img1;
}

Color* palCreate(struct ColorCubeList* first)
{
	struct ColorCubeList* cur;
	int pal_size = 256;
	Color* pal; 
	int i;

	pal = (Color *)calloc(pal_size,sizeof(Color));

    for(cur = first, i = 0;cur != NULL; i++)
    {
        struct ColorCubeList *tmp;

        if (i < pal_size)
		{
            pal[i].r = cur->c->med.r;
            pal[i].g = cur->c->med.g;
            pal[i].b = cur->c->med.b;
		}

        free(cur->c);
        tmp = cur->next;
        free(cur);
        cur = tmp;
    } 

	return pal;
}

Image* bestColor(Image *img0, Color* pal)
{
    int w = imgGetWidth(img0);
    int h = imgGetHeight(img0);
    Image * img1=imgCreate(w,h);
	int pal_size = 256;
	float rgb[3];
	int x,y,i;
	
	for (y=0;y<h;y++){
        for (x=0;x<w;x++){
            int i_menor;
            float m_menor;
			float rgb1[3];
            
            imgGetPixel3fv(img0,x,y,rgb);

            m_menor = FLT_MAX;
            i_menor = 0;

            for(i=0;i<pal_size;i++)
            {
                float dif[3];
                float m;

                dif[0] = rgb[0] - pal[i].r;
                dif[1] = rgb[1] - pal[i].g;
                dif[2] = rgb[2] - pal[i].b;

                m = (float)sqrt( (dif[0]*dif[0]) + (dif[1]*dif[1]) + (dif[2]*dif[2]) );

                if (m < m_menor)
                {
                    i_menor = i;
                    m_menor = m;

					rgb1[0]=pal[i_menor].r;
					rgb1[1]=pal[i_menor].g;
					rgb1[2]=pal[i_menor].b;
                }
            }

            imgSetPixel3fv(img1,x,y,rgb1);
        }
    }
	return img1;
}

Image * imgReduce256Colors(Image * img0)
{
    int w = imgGetWidth(img0);
    int h = imgGetHeight(img0);
    Image * img1=imgCreate(w,h);
    float rgb[3];
	Color * pal;
    int i,stop,x,y;
    struct ColorCubeList *first;
    struct ColorCubeList *cur;
    struct ColorCubeList *tmp;
    int pal_size = 256;

    pal = (Color *)calloc(pal_size,sizeof(Color));

	/* cria o cubo inicial */
    first = malloc(sizeof(struct ColorCubeList));
    first->c = (ColorCube *)malloc(sizeof(ColorCube));
    first->next = NULL;
    first->c->max.r = first->c->max.g = first->c->max.b = 1;
    first->c->min.r = first->c->min.g = first->c->min.b = 0;
    first->c->checked = 0;
    first->c->level = 0;
    first->prev = NULL;

	/* cria a paleta de cores */
    for(i=0;;)
    {
        struct ColorCubeList *cur_maior;
            
        for(cur = first;cur != NULL;cur = cur->next)
        {
            ColorCube init_c;
            float sumrgb[3];

            if (cur->c->checked) continue;

            memcpy(&init_c, cur->c, sizeof(ColorCube));
            cur->c->max.r = cur->c->max.g = cur->c->max.b = 0;
            cur->c->min.r = cur->c->min.g = cur->c->min.b = 1;
            cur->c->count = 0;
            sumrgb[0] = sumrgb[1] = sumrgb[2] = 0;

            for (y=0;y<h;y++)
            {
                for (x=0;x<w;x++)
                {
                    imgGetPixel3fv(img0,x,y,rgb);

                    if ((init_c.max.r >= rgb[0])&&(init_c.max.g >= rgb[1])&&(init_c.max.b >= rgb[2])&&
                        (init_c.min.r <= rgb[0])&&(init_c.min.g <= rgb[1])&&(init_c.min.b <= rgb[2]))
                    {
                        sumrgb[0] += rgb[0];
                        sumrgb[1] += rgb[1];
                        sumrgb[2] += rgb[2];
                        cur->c->count++;
                
                        if (rgb[0] > cur->c->max.r) cur->c->max.r = rgb[0];
                        if (rgb[1] > cur->c->max.g) cur->c->max.g = rgb[1];
                        if (rgb[2] > cur->c->max.b) cur->c->max.b = rgb[2];

                        if (rgb[0] < cur->c->min.r) cur->c->min.r = rgb[0];
                        if (rgb[1] < cur->c->min.g) cur->c->min.g = rgb[1];
                        if (rgb[2] < cur->c->min.b) cur->c->min.b = rgb[2];
                    }
                }
            }

            if (cur->c->count == 0)
            {
                if (cur->prev != NULL) cur->prev->next = cur->next;
                else first = cur->next;
                if   (cur->next != NULL) tmp = cur->next->prev = cur->prev;
                else tmp = NULL;

                i--;
                printf("cube empty, deleting\n");

                free(cur->c);
                free(cur);

                cur = tmp;

                continue;
            }

            cur->c->med.r = sumrgb[0]/cur->c->count;
            cur->c->med.g = sumrgb[1]/cur->c->count;
            cur->c->med.b = sumrgb[2]/cur->c->count;

			/* pega a diferenca entre os maximos e minimos de cada componente */
			cur->c->var1.r = cur->c->max.r - cur->c->min.r;
			cur->c->var1.g = cur->c->max.g - cur->c->min.g;
			cur->c->var1.b = cur->c->max.b - cur->c->min.b;

            cur->c->checked = 1;
		}

		if (i>=pal_size) break;

		/* escolhe o cubo a ser cortado */
        for(cur_maior = cur = first, stop = -1;cur != NULL;cur = cur->next)
        {
			/* cubo com uma unica cor */
            if ((cur->c->max.r == cur->c->min.r)&&(cur->c->max.g == cur->c->min.g)&&(cur->c->max.b == cur->c->min.b))
            {
                    if (stop == -1) stop = 1;
                    continue;
            } else stop = 0;

			if (((cur->c->var1.r > cur_maior->c->var1.r)&&(cur->c->var1.r > cur_maior->c->var1.g)&&(cur->c->var1.r > cur_maior->c->var1.b))||
				((cur->c->var1.g > cur_maior->c->var1.r)&&(cur->c->var1.g > cur_maior->c->var1.g)&&(cur->c->var1.g > cur_maior->c->var1.b))||
				((cur->c->var1.b > cur_maior->c->var1.r)&&(cur->c->var1.b > cur_maior->c->var1.g)&&(cur->c->var1.b > cur_maior->c->var1.b)))
					cur_maior = cur;
        }

        if (stop==1) break;

		/* divide o cubo */
        cur = cur_maior;
        
        tmp = malloc(sizeof(struct ColorCubeList));
        tmp->c = (ColorCube *)malloc(sizeof(ColorCube));

        tmp->c->max.r = cur->c->max.r; tmp->c->min.r = cur->c->min.r;
        tmp->c->max.g = cur->c->max.g; tmp->c->min.g = cur->c->min.g;
        tmp->c->max.b = cur->c->max.b; tmp->c->min.b = cur->c->min.b;

        tmp->next = cur->next;
        cur->next = tmp;
        tmp->prev = cur;

        cur->c->level++;
        tmp->c->level = cur->c->level;

        if ((cur->c->max.r > cur->c->min.r)&&(cur->c->var1.r >= cur->c->var1.g)&&(cur->c->var1.r >= cur->c->var1.b))
        {
            cur->c->max.r = cur->c->med.r;
            tmp->c->min.r = cur->c->med.r + 0.000001f;
        } else
        if ((cur->c->max.g > cur->c->min.g)&&(cur->c->var1.g >= cur->c->var1.r)&&(cur->c->var1.g >= cur->c->var1.b))
        {
            cur->c->max.g = cur->c->med.g;
            tmp->c->min.g = cur->c->med.g + 0.000001f;
        } else
        if ((cur->c->max.b > cur->c->min.b)&&(cur->c->var1.b >= cur->c->var1.r)&&(cur->c->var1.b >= cur->c->var1.g))
        {
            cur->c->max.b = cur->c->med.b;
            tmp->c->min.b = cur->c->med.b + 0.000001f;
        }

        cur->c->checked = 0;
        tmp->c->checked = 0;

		i++;
    }

	/* cria um vetor com as cores da paleta */
	pal = palCreate(first);

	/* cria a figura quantizada usando a paleta de cores */
	img1 = bestColor(img0, pal);

    free(pal);

    return img1;
}

void imgSub(Image *img0, Image *img1)
{
	int w = imgGetWidth(img0);
	int h = imgGetHeight(img0);
	int x,y;
	float rgb0[3],rgb1[3];

	for (y=0;y<h;y++){
		for (x=0;x<w;x++) {
			imgGetPixel3fv(img0,x,y,rgb0);
			imgGetPixel3fv(img1,x,y,rgb1);
				rgb0[0]=(rgb1[0]>rgb0[0])? rgb1[0]-rgb0[0] : rgb0[0]-rgb1[0] ;
				rgb0[1]=(rgb1[1]>rgb0[1])? rgb1[1]-rgb0[1] : rgb0[1]-rgb1[1] ;
				rgb0[2]=(rgb1[2]>rgb0[2])? rgb1[2]-rgb0[2] : rgb0[2]-rgb1[2] ;
			imgSetPixel3fv(img0,x,y,rgb0);
		}
	}
}

Image * imgGaussFilter(Image * img0)
{
	int w = imgGetWidth (img0);
	int h = imgGetHeight(img0);
	Image * img1 = imgCreate(w,h);

	float rgb00[3], rgb01[3], rgb02[3];
	float rgb10[3], rgb11[3], rgb12[3];
	float rgb20[3], rgb21[3], rgb22[3];
	float rgb[3];

	int Gauss[3][3];

	int x, y;

	/* Filtro Gaussiano */
	Gauss[0][0] = 1; Gauss[0][1] = 2; Gauss[0][2] = 1;
	Gauss[1][0] = 2; Gauss[1][1] = 4; Gauss[1][2] = 2;
	Gauss[2][0] = 1; Gauss[2][1] = 2; Gauss[2][2] = 1;

	for (y=0;y<h;y++){
		for (x=0;x<w;x++) {

			/* tratando as bordas */
			if(((y == 0) && (x == 0))){
				imgGetPixel3fv(img0,x+1,  y,rgb12);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,x+1,y+1,rgb22);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				rgb[0] = (rgb11[0]*Gauss[1][1] + rgb12[0]*Gauss[1][2] + rgb21[0]*Gauss[2][1] + rgb22[0]*Gauss[2][2])/16;
				rgb[1] = (rgb11[1]*Gauss[1][1] + rgb12[1]*Gauss[1][2] + rgb21[1]*Gauss[2][1] + rgb22[1]*Gauss[2][2])/16;
				rgb[2] = (rgb11[2]*Gauss[1][1] + rgb12[2]*Gauss[1][2] + rgb21[2]*Gauss[2][1] + rgb22[2]*Gauss[2][2])/16;
			}

			if(((x == (w-1)) && (y == 0))){
				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,x-1,y+1,rgb20);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				rgb[0] = (rgb11[0]*Gauss[1][1] + rgb10[0]*Gauss[1][0] + rgb20[0]*Gauss[2][0] + rgb21[0]*Gauss[2][1])/16;
				rgb[1] = (rgb11[1]*Gauss[1][1] + rgb10[1]*Gauss[1][0] + rgb20[1]*Gauss[2][0] + rgb21[1]*Gauss[2][1])/16;
				rgb[2] = (rgb11[2]*Gauss[1][1] + rgb10[2]*Gauss[1][0] + rgb20[2]*Gauss[2][0] + rgb21[2]*Gauss[2][1])/16;
			}

			if(((x == (w-1)) && (y == (h-1)))){
				imgGetPixel3fv(img0,x-1,y-1,rgb00);
				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				rgb[0] = (rgb11[0]*Gauss[1][1] + rgb00[0]*Gauss[0][0] + rgb01[0]*Gauss[0][1] + rgb10[0]*Gauss[1][0])/16;
				rgb[1] = (rgb11[1]*Gauss[1][1] + rgb00[1]*Gauss[0][0] + rgb01[1]*Gauss[0][1] + rgb10[1]*Gauss[1][0])/16;
				rgb[2] = (rgb11[2]*Gauss[1][1] + rgb00[2]*Gauss[0][0] + rgb01[2]*Gauss[0][1] + rgb10[2]*Gauss[1][0])/16;
			}

			if(((y == (h-1)) && (x == 0))){
				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x+1,y-1,rgb02);
				imgGetPixel3fv(img0,x+1,  y,rgb12);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				rgb[0] = (rgb11[0]*Gauss[1][1] + rgb12[0]*Gauss[1][2] + rgb01[0]*Gauss[0][1] + rgb02[0]*Gauss[0][2])/16;
				rgb[1] = (rgb11[1]*Gauss[1][1] + rgb12[1]*Gauss[1][2] + rgb01[1]*Gauss[0][1] + rgb02[1]*Gauss[0][2])/16;
				rgb[2] = (rgb11[2]*Gauss[1][1] + rgb12[2]*Gauss[1][2] + rgb01[2]*Gauss[0][1] + rgb02[2]*Gauss[0][2])/16;
			}

			if(((x == (w-1)) && (y>0) && (y<(h-1)))){
				imgGetPixel3fv(img0,x-1,y-1,rgb00);
				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,x-1,y+1,rgb20);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				rgb[0] = (rgb11[0]*Gauss[1][1] + rgb00[0]*Gauss[0][0] + rgb01[0]*Gauss[0][1] + rgb10[0]*Gauss[1][0] + rgb21[0]*Gauss[2][1] + rgb20[0]*Gauss[2][0])/16;
				rgb[1] = (rgb11[1]*Gauss[1][1] + rgb00[1]*Gauss[0][0] + rgb01[1]*Gauss[0][1] + rgb10[1]*Gauss[1][0] + rgb21[1]*Gauss[2][1] + rgb20[1]*Gauss[2][0])/16;
				rgb[2] = (rgb11[2]*Gauss[1][1] + rgb00[2]*Gauss[0][0] + rgb01[2]*Gauss[0][1] + rgb10[2]*Gauss[1][0] + rgb21[2]*Gauss[2][1] + rgb20[2]*Gauss[2][0])/16;
			}

			if(((x == 0) && (y > 0) && (y < (h-1)))){
				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x+1,y-1,rgb02);
				imgGetPixel3fv(img0,x+1,  y,rgb12);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,x+1,y+1,rgb22);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				rgb[0] = (rgb11[0]*Gauss[1][1] + rgb01[0]*Gauss[0][1] + rgb02[0]*Gauss[0][2] + rgb12[0]*Gauss[1][2] + rgb21[0]*Gauss[2][1] + rgb22[0]*Gauss[2][2])/16;
				rgb[1] = (rgb11[1]*Gauss[1][1] + rgb01[1]*Gauss[0][1] + rgb02[1]*Gauss[0][2] + rgb12[1]*Gauss[1][2] + rgb21[1]*Gauss[2][1] + rgb22[1]*Gauss[2][2])/16;
				rgb[2] = (rgb11[2]*Gauss[1][1] + rgb01[2]*Gauss[0][1] + rgb02[2]*Gauss[0][2] + rgb12[2]*Gauss[1][2] + rgb21[2]*Gauss[2][1] + rgb22[2]*Gauss[2][2])/16;
			}

			if(((y == 0) && (x > 0) && (x < (w-1)))){
				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,x+1,  y,rgb12);
				imgGetPixel3fv(img0,x-1,y+1,rgb20);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,x+1,y+1,rgb22);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				rgb[0] = (rgb11[0]*Gauss[1][1] + rgb10[0]*Gauss[1][0] + rgb12[0]*Gauss[1][2] + rgb20[0]*Gauss[2][0] + rgb21[0]*Gauss[2][1] + rgb22[0]*Gauss[2][2])/16;
				rgb[1] = (rgb11[1]*Gauss[1][1] + rgb10[1]*Gauss[1][0] + rgb12[1]*Gauss[1][2] + rgb20[1]*Gauss[2][0] + rgb21[1]*Gauss[2][1] + rgb22[1]*Gauss[2][2])/16;
				rgb[2] = (rgb11[2]*Gauss[1][1] + rgb10[2]*Gauss[1][0] + rgb12[2]*Gauss[1][2] + rgb20[2]*Gauss[2][0] + rgb21[2]*Gauss[2][1] + rgb22[2]*Gauss[2][2])/16;
			}

			if(((y == (h-1)) && (x > 0) && (x < (w-1)))){
				imgGetPixel3fv(img0,x-1,y-1,rgb00);
				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x+1,y-1,rgb02);
				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,  x,  y,rgb11);
				imgGetPixel3fv(img0,x+1,  y,rgb12);

				rgb[0] = (rgb11[0]*Gauss[1][1] + rgb00[0]*Gauss[0][0] + rgb01[0]*Gauss[0][1] + rgb02[0]*Gauss[0][2] + rgb10[0]*Gauss[1][0] + rgb12[0]*Gauss[1][2])/16;
				rgb[1] = (rgb11[1]*Gauss[1][1] + rgb00[1]*Gauss[0][0] + rgb01[1]*Gauss[0][1] + rgb02[1]*Gauss[0][2] + rgb10[1]*Gauss[1][0] + rgb12[1]*Gauss[1][2])/16;
				rgb[2] = (rgb11[2]*Gauss[1][1] + rgb00[2]*Gauss[0][0] + rgb01[2]*Gauss[0][1] + rgb02[2]*Gauss[0][2] + rgb10[2]*Gauss[1][0] + rgb12[2]*Gauss[1][2])/16;
			}

			/* tratando os pixels centrais */
			if( ((x!=0) && (y!=0) && (y!=(h-1)) && (x!=(w-1)))){
				imgGetPixel3fv(img0,x-1,y-1,rgb00);
				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x+1,y-1,rgb02);
				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,  x,  y,rgb11);
				imgGetPixel3fv(img0,x+1,  y,rgb12);
				imgGetPixel3fv(img0,x-1,y+1,rgb20);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,x+1,y+1,rgb22);

				rgb[0] = (rgb00[0]*Gauss[0][0] + rgb01[0]*Gauss[0][1] + rgb02[0]*Gauss[0][2] + 
						  rgb10[0]*Gauss[1][0] + rgb11[0]*Gauss[1][1] + rgb12[0]*Gauss[1][2] + 
						  rgb20[0]*Gauss[2][0] + rgb21[0]*Gauss[2][1] + rgb22[0]*Gauss[2][2])/16;

				rgb[1] = (rgb00[1]*Gauss[0][0] + rgb01[1]*Gauss[0][1] + rgb02[1]*Gauss[0][2] + 
						  rgb10[1]*Gauss[1][0] + rgb11[1]*Gauss[1][1] + rgb12[1]*Gauss[1][2] + 
						  rgb20[1]*Gauss[2][0] + rgb21[1]*Gauss[2][1] + rgb22[1]*Gauss[2][2])/16;

				rgb[2] = (rgb00[2]*Gauss[0][0] + rgb01[2]*Gauss[0][1] + rgb02[2]*Gauss[0][2] + 
						  rgb10[2]*Gauss[1][0] + rgb11[2]*Gauss[1][1] + rgb12[2]*Gauss[1][2] + 
						  rgb20[2]*Gauss[2][0] + rgb21[2]*Gauss[2][1] + rgb22[2]*Gauss[2][2])/16;
			}

			if(rgb[0]>1) rgb[0] = 1;
			if(rgb[1]>1) rgb[1] = 1;
			if(rgb[2]>1) rgb[2] = 1;

			imgSetPixel3fv(img1,x,y,rgb);
		}
	}

	return img1;
}

Image * imgLaplcFilter(Image * img)
{
	int w = imgGetWidth (img);
	int h = imgGetHeight(img);
	Image * img1 = imgCreate(w,h);
	Image * img0 = imgCreate(w,h);

	float rgb00[3], rgb01[3], rgb02[3];
	float rgb10[3], rgb11[3], rgb12[3];
	float rgb20[3], rgb21[3], rgb22[3];
	float rgb[3];

	int Laplc[3][3];

	int x, y;

	img0 = imgGrey(img);

	/* Filtro Laplaciano */
	Laplc[0][0] = -1; Laplc[0][1] = -1; Laplc[0][2] = -1;
	Laplc[1][0] = -1; Laplc[1][1] =  8; Laplc[1][2] = -1;
	Laplc[2][0] = -1; Laplc[2][1] = -1; Laplc[2][2] = -1;

	for (y=0;y<h;y++){
		for (x=0;x<w;x++) {

			/* tratando as bordas */
			if(((y == 0) && (x == 0))){
				imgGetPixel3fv(img0,x+1,  y,rgb12);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,x+1,y+1,rgb22);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				rgb[0] = (rgb11[0]*Laplc[1][1] + rgb12[0]*Laplc[1][2] + rgb21[0]*Laplc[2][1] + rgb22[0]*Laplc[2][2]);
				rgb[1] = (rgb11[1]*Laplc[1][1] + rgb12[1]*Laplc[1][2] + rgb21[1]*Laplc[2][1] + rgb22[1]*Laplc[2][2]);
				rgb[2] = (rgb11[2]*Laplc[1][1] + rgb12[2]*Laplc[1][2] + rgb21[2]*Laplc[2][1] + rgb22[2]*Laplc[2][2]);
			}

			if(((x == (w-1)) && (y == 0))){
				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,x-1,y+1,rgb20);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				rgb[0] = (rgb11[0]*Laplc[1][1] + rgb10[0]*Laplc[1][0] + rgb20[0]*Laplc[2][0] + rgb21[0]*Laplc[2][1]);
				rgb[1] = (rgb11[1]*Laplc[1][1] + rgb10[1]*Laplc[1][0] + rgb20[1]*Laplc[2][0] + rgb21[1]*Laplc[2][1]);
				rgb[2] = (rgb11[2]*Laplc[1][1] + rgb10[2]*Laplc[1][0] + rgb20[2]*Laplc[2][0] + rgb21[2]*Laplc[2][1]);
			}

			if(((x == (w-1)) && (y == (h-1)))){
				imgGetPixel3fv(img0,x-1,y-1,rgb00);
				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				rgb[0] = (rgb11[0]*Laplc[1][1] + rgb00[0]*Laplc[0][0] + rgb01[0]*Laplc[0][1] + rgb10[0]*Laplc[1][0]);
				rgb[1] = (rgb11[1]*Laplc[1][1] + rgb00[1]*Laplc[0][0] + rgb01[1]*Laplc[0][1] + rgb10[1]*Laplc[1][0]);
				rgb[2] = (rgb11[2]*Laplc[1][1] + rgb00[2]*Laplc[0][0] + rgb01[2]*Laplc[0][1] + rgb10[2]*Laplc[1][0]);
			}

			if(((y == (h-1)) && (x == 0))){
				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x+1,y-1,rgb02);
				imgGetPixel3fv(img0,x+1,  y,rgb12);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				rgb[0] = (rgb11[0]*Laplc[1][1] + rgb12[0]*Laplc[1][2] + rgb01[0]*Laplc[0][1] + rgb02[0]*Laplc[0][2]);
				rgb[1] = (rgb11[1]*Laplc[1][1] + rgb12[1]*Laplc[1][2] + rgb01[1]*Laplc[0][1] + rgb02[1]*Laplc[0][2]);
				rgb[2] = (rgb11[2]*Laplc[1][1] + rgb12[2]*Laplc[1][2] + rgb01[2]*Laplc[0][1] + rgb02[2]*Laplc[0][2]);
			}

			if(((x == (w-1)) && (y>0) && (y<(h-1)))){
				imgGetPixel3fv(img0,x-1,y-1,rgb00);
				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,x-1,y+1,rgb20);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				rgb[0] = (rgb11[0]*Laplc[1][1] + rgb00[0]*Laplc[0][0] + rgb01[0]*Laplc[0][1] + rgb10[0]*Laplc[1][0] + rgb21[0]*Laplc[2][1] + rgb20[0]*Laplc[2][0]);
				rgb[1] = (rgb11[1]*Laplc[1][1] + rgb00[1]*Laplc[0][0] + rgb01[1]*Laplc[0][1] + rgb10[1]*Laplc[1][0] + rgb21[1]*Laplc[2][1] + rgb20[1]*Laplc[2][0]);
				rgb[2] = (rgb11[2]*Laplc[1][1] + rgb00[2]*Laplc[0][0] + rgb01[2]*Laplc[0][1] + rgb10[2]*Laplc[1][0] + rgb21[2]*Laplc[2][1] + rgb20[2]*Laplc[2][0]);
			}

			if(((x == 0) && (y > 0) && (y < (h-1)))){
				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x+1,y-1,rgb02);
				imgGetPixel3fv(img0,x+1,  y,rgb12);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,x+1,y+1,rgb22);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				rgb[0] = (rgb11[0]*Laplc[1][1] + rgb01[0]*Laplc[0][1] + rgb02[0]*Laplc[0][2] + rgb12[0]*Laplc[1][2] + rgb21[0]*Laplc[2][1] + rgb22[0]*Laplc[2][2]);
				rgb[1] = (rgb11[1]*Laplc[1][1] + rgb01[1]*Laplc[0][1] + rgb02[1]*Laplc[0][2] + rgb12[1]*Laplc[1][2] + rgb21[1]*Laplc[2][1] + rgb22[1]*Laplc[2][2]);
				rgb[2] = (rgb11[2]*Laplc[1][1] + rgb01[2]*Laplc[0][1] + rgb02[2]*Laplc[0][2] + rgb12[2]*Laplc[1][2] + rgb21[2]*Laplc[2][1] + rgb22[2]*Laplc[2][2]);
			}

			if(((y == 0) && (x > 0) && (x < (w-1)))){
				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,x+1,  y,rgb12);
				imgGetPixel3fv(img0,x-1,y+1,rgb20);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,x+1,y+1,rgb22);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				rgb[0] = (rgb11[0]*Laplc[1][1] + rgb10[0]*Laplc[1][0] + rgb12[0]*Laplc[1][2] + rgb20[0]*Laplc[2][0] + rgb21[0]*Laplc[2][1] + rgb22[0]*Laplc[2][2]);
				rgb[1] = (rgb11[1]*Laplc[1][1] + rgb10[1]*Laplc[1][0] + rgb12[1]*Laplc[1][2] + rgb20[1]*Laplc[2][0] + rgb21[1]*Laplc[2][1] + rgb22[1]*Laplc[2][2]);
				rgb[2] = (rgb11[2]*Laplc[1][1] + rgb10[2]*Laplc[1][0] + rgb12[2]*Laplc[1][2] + rgb20[2]*Laplc[2][0] + rgb21[2]*Laplc[2][1] + rgb22[2]*Laplc[2][2]);
			}

			if(((y == (h-1)) && (x > 0) && (x < (w-1)))){
				imgGetPixel3fv(img0,x-1,y-1,rgb00);
				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x+1,y-1,rgb02);
				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,  x,  y,rgb11);
				imgGetPixel3fv(img0,x+1,  y,rgb12);

				rgb[0] = (rgb11[0]*Laplc[1][1] + rgb00[0]*Laplc[0][0] + rgb01[0]*Laplc[0][1] + rgb02[0]*Laplc[0][2] + rgb10[0]*Laplc[1][0] + rgb12[0]*Laplc[1][2]);
				rgb[1] = (rgb11[1]*Laplc[1][1] + rgb00[1]*Laplc[0][0] + rgb01[1]*Laplc[0][1] + rgb02[1]*Laplc[0][2] + rgb10[1]*Laplc[1][0] + rgb12[1]*Laplc[1][2]);
				rgb[2] = (rgb11[2]*Laplc[1][1] + rgb00[2]*Laplc[0][0] + rgb01[2]*Laplc[0][1] + rgb02[2]*Laplc[0][2] + rgb10[2]*Laplc[1][0] + rgb12[2]*Laplc[1][2]);
			}

			/* tratando os pixels centrais */
			if( ((x!=0) && (y!=0) && (y!=(h-1)) && (x!=(w-1)))){
				imgGetPixel3fv(img0,x-1,y-1,rgb00);
				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x+1,y-1,rgb02);
				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,  x,  y,rgb11);
				imgGetPixel3fv(img0,x+1,  y,rgb12);
				imgGetPixel3fv(img0,x-1,y+1,rgb20);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,x+1,y+1,rgb22);

				rgb[0] = (rgb00[0]*Laplc[0][0] + rgb01[0]*Laplc[0][1] + rgb02[0]*Laplc[0][2] + 
						  rgb10[0]*Laplc[1][0] + rgb11[0]*Laplc[1][1] + rgb12[0]*Laplc[1][2] + 
						  rgb20[0]*Laplc[2][0] + rgb21[0]*Laplc[2][1] + rgb22[0]*Laplc[2][2]);

				rgb[1] = (rgb00[1]*Laplc[0][0] + rgb01[1]*Laplc[0][1] + rgb02[1]*Laplc[0][2] + 
						  rgb10[1]*Laplc[1][0] + rgb11[1]*Laplc[1][1] + rgb12[1]*Laplc[1][2] + 
						  rgb20[1]*Laplc[2][0] + rgb21[1]*Laplc[2][1] + rgb22[1]*Laplc[2][2]);

				rgb[2] = (rgb00[2]*Laplc[0][0] + rgb01[2]*Laplc[0][1] + rgb02[2]*Laplc[0][2] + 
						  rgb10[2]*Laplc[1][0] + rgb11[2]*Laplc[1][1] + rgb12[2]*Laplc[1][2] + 
						  rgb20[2]*Laplc[2][0] + rgb21[2]*Laplc[2][1] + rgb22[2]*Laplc[2][2]);
			}

			if(rgb[0]>1) rgb[0] = 1;
			if(rgb[1]>1) rgb[1] = 1;
			if(rgb[2]>1) rgb[2] = 1;

			if(rgb[0]<0) rgb[0] = 0;
			if(rgb[1]<0) rgb[1] = 0;
			if(rgb[2]<0) rgb[2] = 0;

			imgSetPixel3fv(img1,x,y,rgb);
		}
	}

	return img1;
}

Image * imgEdges(Image * img0)
{
	int w = imgGetWidth (img0);
	int h = imgGetHeight(img0);
	Image * imgCinza = imgCreate(w,h);
	Image * imgGauss = imgCreate(w,h);
	Image * imgLoG   = imgCreate(w,h);
	Image * imgLoG1  = imgCreate(w,h);

	float rgb1[3];
	float rgb[3];

	int x, y;

	imgGauss = imgGaussFilter(img0);
	imgLoG   = imgLaplcFilter(imgGauss);

	for (y=0;y<h;y++){
		for (x=0;x<w;x++) {

			imgGetPixel3fv(imgLoG,x,y,rgb);

				rgb1[0] = 1 - rgb[0];
				rgb1[1] = 1 - rgb[1];
				rgb1[2] = 1 - rgb[2];

			imgSetPixel3fv(imgLoG1,x,y,rgb1);
		}
	}

	return imgLoG1;
}




Image * imgFiltroDeMediana(Image * img0)
{
	int w = imgGetWidth (img0);
	int h = imgGetHeight(img0);
	Image * img1 = imgCreate(w,h);

	float rgb00[3], rgb01[3], rgb02[3];
	float rgb10[3], rgb11[3], rgb12[3];
	float rgb20[3], rgb21[3], rgb22[3];
	float rgb[3];

	int x, y;

	for (y=2;y<h-2;y++){
		for (x=2;x<w-2;x++) {

			/* tratando as bordas */
			if(((y == 0) && (x == 0))){

				float vet0[4]; float vet1[4]; float vet2[4];

				imgGetPixel3fv(img0,x+1,  y,rgb12);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,x+1,y+1,rgb22);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				vet0[0] = rgb11[0];	vet0[2] = rgb21[0];
				vet0[1] = rgb12[0];	vet0[3] = rgb22[0];
			
				vet1[0] = rgb11[0];	vet1[2] = rgb21[0];
				vet1[1] = rgb12[0];	vet1[3] = rgb22[0];

				vet2[0] = rgb11[0];	vet2[2] = rgb21[0];
				vet2[1] = rgb12[0];	vet2[3] = rgb22[0];

				qsort(vet0,4,sizeof(float),comparaCor);
				qsort(vet1,4,sizeof(float),comparaCor);
				qsort(vet2,4,sizeof(float),comparaCor);

				rgb[0] = vet0[2];
				rgb[1] = vet1[2];
				rgb[2] = vet2[2];
			}

			if(((x == (w-1)) && (y == 0))){

				float vet0[4]; float vet1[4]; float vet2[4];

				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,x-1,y+1,rgb20);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				vet0[0] = rgb10[0];	vet0[2] = rgb21[0];
				vet0[1] = rgb20[0];	vet0[3] = rgb11[0];
			
				vet1[0] = rgb10[0];	vet1[2] = rgb21[0];
				vet1[1] = rgb20[0];	vet1[3] = rgb11[0];

				vet2[0] = rgb10[0];	vet2[2] = rgb21[0];
				vet2[1] = rgb20[0];	vet2[3] = rgb11[0];

				qsort(vet0,4,sizeof(float),comparaCor);
				qsort(vet1,4,sizeof(float),comparaCor);
				qsort(vet2,4,sizeof(float),comparaCor);

				rgb[0] = vet0[2];
				rgb[1] = vet1[2];
				rgb[2] = vet2[2];
			}

			if(((x == (w-1)) && (y == (h-1)))){

				float vet0[4]; float vet1[4]; float vet2[4];

				imgGetPixel3fv(img0,x-1,y-1,rgb00);
				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				vet0[0] = rgb00[0];	vet0[2] = rgb10[0];
				vet0[1] = rgb01[0];	vet0[3] = rgb11[0];
			
				vet1[0] = rgb00[0];	vet1[2] = rgb10[0];
				vet1[1] = rgb01[0];	vet1[3] = rgb11[0];

				vet2[0] = rgb00[0];	vet2[2] = rgb10[0];
				vet2[1] = rgb01[0];	vet2[3] = rgb11[0];

				qsort(vet0,4,sizeof(float),comparaCor);
				qsort(vet1,4,sizeof(float),comparaCor);
				qsort(vet2,4,sizeof(float),comparaCor);

				rgb[0] = vet0[2];
				rgb[1] = vet1[2];
				rgb[2] = vet2[2];
			}

			if(((y == (h-1)) && (x == 0))){

				float vet0[4]; float vet1[4]; float vet2[4];

				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x+1,y-1,rgb02);
				imgGetPixel3fv(img0,x+1,  y,rgb12);
				imgGetPixel3fv(img0,  x,  y,rgb11);

				vet0[0] = rgb01[0];	vet0[2] = rgb11[0];
				vet0[1] = rgb02[0];	vet0[3] = rgb12[0];
			
				vet1[0] = rgb01[0];	vet1[2] = rgb11[0];
				vet1[1] = rgb02[0];	vet1[3] = rgb12[0];

				vet2[0] = rgb01[0];	vet2[2] = rgb11[0];
				vet2[1] = rgb02[0];	vet2[3] = rgb12[0];

				qsort(vet0,4,sizeof(float),comparaCor);
				qsort(vet1,4,sizeof(float),comparaCor);
				qsort(vet2,4,sizeof(float),comparaCor);

				rgb[0] = vet0[2];
				rgb[1] = vet1[2];
				rgb[2] = vet2[2];
			}

			if(((x == (w-1)) && (y>0) && (y<(h-1)))){

				float vet0[6]; float vet1[6]; float vet2[6];

				imgGetPixel3fv(img0,x-1,y-1,rgb00);
				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,  x,  y,rgb11);
				imgGetPixel3fv(img0,x-1,y+1,rgb20);
				imgGetPixel3fv(img0,  x,y+1,rgb21);

				vet0[0] = rgb00[0];	vet0[3] = rgb11[0];
				vet0[1] = rgb01[0];	vet0[4] = rgb20[0];
				vet0[2] = rgb10[0];	vet0[5] = rgb21[0];
			
				vet1[0] = rgb00[0];	vet1[3] = rgb11[0];
				vet1[1] = rgb01[0];	vet1[4] = rgb20[0];
				vet1[2] = rgb10[0];	vet1[5] = rgb21[0];

				vet2[0] = rgb00[0];	vet2[3] = rgb11[0];
				vet2[1] = rgb01[0];	vet2[4] = rgb20[0];
				vet2[2] = rgb10[0];	vet2[5] = rgb21[0];

				qsort(vet0,6,sizeof(float),comparaCor);
				qsort(vet1,6,sizeof(float),comparaCor);
				qsort(vet2,6,sizeof(float),comparaCor);

				rgb[0] = vet0[3];
				rgb[1] = vet1[3];
				rgb[2] = vet2[3];
			}

			if(((x == 0) && (y > 0) && (y < (h-1)))){

				float vet0[6]; float vet1[6]; float vet2[6];

				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x+1,y-1,rgb02);
				imgGetPixel3fv(img0,  x,  y,rgb11);
				imgGetPixel3fv(img0,x+1,  y,rgb12);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,x+1,y+1,rgb22);

				vet0[0] = rgb01[0];	vet0[3] = rgb12[0];
				vet0[1] = rgb02[0];	vet0[4] = rgb21[0];
				vet0[2] = rgb11[0];	vet0[5] = rgb22[0];

				vet1[0] = rgb01[0];	vet1[3] = rgb12[0];
				vet1[1] = rgb02[0];	vet1[4] = rgb21[0];
				vet1[2] = rgb11[0];	vet1[5] = rgb22[0];

				vet2[0] = rgb01[0];	vet2[3] = rgb12[0];
				vet2[1] = rgb02[0];	vet2[4] = rgb21[0];
				vet2[2] = rgb11[0];	vet2[5] = rgb22[0];

				qsort(vet0,6,sizeof(float),comparaCor);
				qsort(vet1,6,sizeof(float),comparaCor);
				qsort(vet2,6,sizeof(float),comparaCor);

				rgb[0] = vet0[3];
				rgb[1] = vet1[3];
				rgb[2] = vet2[3];
			}

			if(((y == 0) && (x > 0) && (x < (w-1)))){

				float vet0[6]; float vet1[6]; float vet2[6];

				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,  x,  y,rgb11);
				imgGetPixel3fv(img0,x+1,  y,rgb12);
				imgGetPixel3fv(img0,x-1,y+1,rgb20);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,x+1,y+1,rgb22);

				vet0[0] = rgb10[0];	vet0[3] = rgb20[0];
				vet0[1] = rgb11[0];	vet0[4] = rgb21[0];
				vet0[2] = rgb12[0];	vet0[5] = rgb22[0];

				vet1[0] = rgb10[1];	vet1[3] = rgb20[1];
				vet1[1] = rgb11[1];	vet1[4] = rgb21[1];
				vet1[2] = rgb12[1];	vet1[5] = rgb22[1];

				vet2[0] = rgb10[2];	vet2[3] = rgb20[2];
				vet2[1] = rgb11[2];	vet2[4] = rgb21[2];
				vet2[2] = rgb12[2];	vet2[5] = rgb22[2];

				qsort(vet0,6,sizeof(float),comparaCor);
				qsort(vet1,6,sizeof(float),comparaCor);
				qsort(vet2,6,sizeof(float),comparaCor);

				rgb[0] = vet0[3];
				rgb[1] = vet1[3];
				rgb[2] = vet2[3];
			}

			if(((y == (h-1)) && (x > 0) && (x < (w-1)))){

				float vet0[6]; float vet1[6]; float vet2[6];

				imgGetPixel3fv(img0,x-1,y-1,rgb00);
				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x+1,y-1,rgb02);
				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,  x,  y,rgb11);
				imgGetPixel3fv(img0,x+1,  y,rgb12);

				vet0[0] = rgb00[0];  vet0[3] = rgb10[0];
				vet0[1] = rgb01[0];  vet0[4] = rgb11[0];
				vet0[2] = rgb02[0];  vet0[5] = rgb12[0];

				vet1[0] = rgb00[1];  vet1[3] = rgb10[1];
				vet1[1] = rgb01[1];  vet1[4] = rgb11[1];
				vet1[2] = rgb02[1];  vet1[5] = rgb12[1];

				vet2[0] = rgb00[2];  vet2[3] = rgb10[2];
				vet2[1] = rgb01[2];  vet2[4] = rgb11[2];
				vet2[2] = rgb02[2];  vet2[5] = rgb12[2];

				qsort(vet0,6,sizeof(float),comparaCor);
				qsort(vet1,6,sizeof(float),comparaCor);
				qsort(vet2,6,sizeof(float),comparaCor);

				rgb[0] = vet0[3];
				rgb[1] = vet1[3];
				rgb[2] = vet2[3];
			}

			/* tratando os pixels centrais */
			if( ((x!=0) && (y!=0) && (y!=(h-1)) && (x!=(w-1)))){

				float vet0[9]; float vet1[9]; float vet2[9];

				imgGetPixel3fv(img0,x-1,y-1,rgb00);
				imgGetPixel3fv(img0,  x,y-1,rgb01);
				imgGetPixel3fv(img0,x+1,y-1,rgb02);
				imgGetPixel3fv(img0,x-1,  y,rgb10);
				imgGetPixel3fv(img0,  x,  y,rgb11);
				imgGetPixel3fv(img0,x+1,  y,rgb12);
				imgGetPixel3fv(img0,x-1,y+1,rgb20);
				imgGetPixel3fv(img0,  x,y+1,rgb21);
				imgGetPixel3fv(img0,x+1,y+1,rgb22);

				vet0[0] = rgb00[0];  vet0[3] = rgb10[0];	vet0[6] = rgb20[0];
				vet0[1] = rgb01[0];  vet0[4] = rgb11[0];	vet0[7] = rgb21[0];
				vet0[2] = rgb02[0];  vet0[5] = rgb12[0];	vet0[8] = rgb22[0];

				vet1[0] = rgb00[1];  vet1[3] = rgb10[1];	vet1[6] = rgb20[1];
				vet1[1] = rgb01[1];  vet1[4] = rgb11[1];	vet1[7] = rgb21[1];
				vet1[2] = rgb02[1];  vet1[5] = rgb12[1];	vet1[8] = rgb22[1];

				vet2[0] = rgb00[2];  vet2[3] = rgb10[2];	vet2[6] = rgb20[2];
				vet2[1] = rgb01[2];  vet2[4] = rgb11[2];	vet2[7] = rgb21[2];
				vet2[2] = rgb02[2];  vet2[5] = rgb12[2];	vet2[8] = rgb22[2];

				qsort(vet0,9,sizeof(float),comparaCor);
				qsort(vet1,9,sizeof(float),comparaCor);
				qsort(vet2,9,sizeof(float),comparaCor);

				rgb[0] = vet0[5];
				rgb[1] = vet1[5];
				rgb[2] = vet2[5];
			}

			if(rgb[0]>1) rgb[0] = 1;
			if(rgb[1]>1) rgb[1] = 1;
			if(rgb[2]>1) rgb[2] = 1;

			imgSetPixel3fv(img1,x,y,rgb);
		}
	}

	return img1;
}

Image * imgFiltroDeMedianaH(Image * img0)
{
	int w = imgGetWidth (img0);
	int h = imgGetHeight(img0);
	Image * img1 = imgCreate(w,h);

	float rgb00[3], rgb02[3], rgb04[3];
	float rgb11[3], rgb12[3], rgb13[3];
	float rgb20[3], rgb21[3], rgb22[3], rgb23[3], rgb24[3];
	float rgb31[3], rgb32[3], rgb33[3];
	float rgb40[3], rgb42[3], rgb44[3];
	float rgb[3];

	int x, y;

	for (y=2;y<h-2;y++){
		for (x=2;x<w-2;x++) {

			/* tratando os pixels centrais */
			if( ((x!=0) && (y!=0) && (y!=(h-1)) && (x!=(w-1)))){

				float vet0[8], vet1[8], vet2[8], vet3[8], vet4[8], vet5[8], vet6[3], vet7[3], vet8[3];

				/* pixels pertencentes a zona 1 */
				imgGetPixel3fv(img0,x-2,y-2,rgb00);
				imgGetPixel3fv(img0,x+2,y-2,rgb04);
				imgGetPixel3fv(img0,x-1,y-1,rgb11);
				imgGetPixel3fv(img0,x+1,y-1,rgb13);
				imgGetPixel3fv(img0,x-1,y+1,rgb31);
				imgGetPixel3fv(img0,x+1,y+1,rgb33);
				imgGetPixel3fv(img0,x-2,y+2,rgb40);
				imgGetPixel3fv(img0,x+2,y+2,rgb44);

				/* ordena os pixels da zona 1 e escolhe o valor mediano */
				vet0[0] = rgb00[0]; vet0[1] = rgb04[0]; vet0[2] = rgb11[0]; vet0[3] = rgb13[0];
				vet0[4] = rgb31[0]; vet0[5] = rgb33[0]; vet0[6] = rgb40[0]; vet0[7] = rgb44[0];

				vet1[0] = rgb00[1]; vet1[1] = rgb04[1]; vet1[2] = rgb11[1]; vet1[3] = rgb13[1];
				vet1[4] = rgb31[1]; vet1[5] = rgb33[1]; vet1[6] = rgb40[1]; vet1[7] = rgb44[1];

				vet2[0] = rgb00[2]; vet2[1] = rgb04[2]; vet2[2] = rgb11[2]; vet2[3] = rgb13[2];
				vet2[4] = rgb31[2]; vet2[5] = rgb33[2]; vet2[6] = rgb40[2]; vet2[7] = rgb44[2];

				qsort(vet0,8,sizeof(float),comparaCor);
				qsort(vet1,8,sizeof(float),comparaCor);
				qsort(vet2,8,sizeof(float),comparaCor);

				/* pixels pertencentes a zona 2 */
				imgGetPixel3fv(img0,  x,y-2,rgb02);
				imgGetPixel3fv(img0,  x,y-1,rgb12);
				imgGetPixel3fv(img0,x-2,y,rgb20);
				imgGetPixel3fv(img0,x-1,y,rgb21);
				imgGetPixel3fv(img0,x+1,y,rgb23);
				imgGetPixel3fv(img0,x+2,y,rgb24);
				imgGetPixel3fv(img0,  x,y+1,rgb32);
				imgGetPixel3fv(img0,  x,y+2,rgb42);

				/* ordena os pixels da zona 2 e escolhe o valor mediano */

				vet3[0] = rgb02[0]; vet3[1] = rgb12[0]; vet3[2] = rgb20[0]; vet3[3] = rgb21[0];
				vet3[4] = rgb23[0]; vet3[5] = rgb24[0]; vet3[6] = rgb32[0]; vet3[7] = rgb42[0];

				vet4[0] = rgb02[1]; vet4[1] = rgb12[1]; vet4[2] = rgb20[1]; vet4[3] = rgb21[1];
				vet4[4] = rgb23[1]; vet4[5] = rgb24[1]; vet4[6] = rgb32[1]; vet4[7] = rgb42[1];

				vet5[0] = rgb02[2]; vet5[1] = rgb12[2]; vet5[2] = rgb20[2]; vet5[3] = rgb21[2];
				vet5[4] = rgb23[2]; vet5[5] = rgb24[2]; vet5[6] = rgb32[2]; vet5[7] = rgb42[2];

				qsort(vet3,8,sizeof(float),comparaCor);
				qsort(vet4,8,sizeof(float),comparaCor);
				qsort(vet5,8,sizeof(float),comparaCor);

				/* ordena os medianos de cada zona e o pixel central */

				imgGetPixel3fv(img0,  x,y,rgb22);

				vet6[0] = vet0[3]; vet6[1] = vet3[3]; vet6[2] = rgb22[0];
				vet7[0] = vet1[3]; vet7[1] = vet4[3]; vet7[2] = rgb22[1];
				vet8[0] = vet2[3]; vet8[1] = vet5[3]; vet8[2] = rgb22[2];

				qsort(vet6,3,sizeof(float),comparaCor);
				qsort(vet7,3,sizeof(float),comparaCor);
				qsort(vet8,3,sizeof(float),comparaCor);

				rgb[0] = vet6[1];
				rgb[1] = vet7[1];
				rgb[2] = vet8[1];
			}

			if(rgb[0]>1) rgb[0] = 1;
			if(rgb[1]>1) rgb[1] = 1;
			if(rgb[2]>1) rgb[2] = 1;

			imgSetPixel3fv(img1,x,y,rgb);
		}
	}

	return img1;
}

Image * imgBinarizacao(Image * img0)
{
	int w = imgGetWidth (img0);
	int h = imgGetHeight(img0);
	float rgb[3], rgbB[3];
	float sumR = 0;
	float sumG = 0;
	float sumB = 0;

	int x, y;

	img0 = imgGrey(img0);

	for (y=0;y<h;y++){
		for (x=0;x<w;x++) {

			imgGetPixel3fv(img0,x,y,rgb);

			sumR += rgb[0];
			sumG += rgb[0];
			sumB += rgb[0];
		}
	}

	for (y=0;y<h;y++){
		for (x=0;x<w;x++) {

			imgGetPixel3fv(img0,x,y,rgb);

			if(rgb[0] > (sumR / (w*h))) rgbB[0] = 1;
			else rgbB[0] = 0;
			if(rgb[1] > (sumG / (w*h))) rgbB[1] = 1;
			else rgbB[1] = 0;
			if(rgb[2] > (sumB / (w*h))) rgbB[2] = 1;
			else rgbB[2] = 0;

			imgSetPixel3fv(img0,x,y,rgbB);
		}
	}

	return img0;
}

Image * imgCopy(Image * image)
{
	int w = imgGetWidth(image);
	int h = imgGetHeight(image);
	Image * img1=imgCreate(w,h);
	int x,y;
	float rgb[3];
   //unsigned char rgb[3];


	for (y=0;y<h;y++){
		for (x=0;x<w;x++) {
			imgGetPixel3fv(image,x,y,rgb);
			imgSetPixel3fv(img1,x,y,rgb);
			//imgGetPixel3ubv(image,x,y,rgb);
			//imgSetPixel3ubv(img1,x,y,rgb);
		}
	}

	return img1;
}
