/**
 * Image.h
 * 
 *
 * @authors $Author: $ - Malf
 * @version $Id: $
 */


/**************************************************************/
/*          BMP FILE STRUCTURE (DEFINITION)				   */
/**************************************************************/
/*  File Header Structure.															   */
/*  2   Type;       File Type Identifier			           					*/
/*  4   FileSize;   Size of File														*/
/*  2   Reserved1;  Reserved (should be 0)						   		*/
/*  2   Reserved2;  Reserved (should be 0)					     	   */
/*  4   Offset;     Offset to bitmap data            					   */
/*  14  - Total File Header Size in Bytes									*/
/*																											*/
/*  Information Header Structure. 												*/
/*  4  Size;            Size of Remaining Header							*/
/*  4  Width;           Width of Bitmap in Pixels							*/
/*  4  Height;          Height of Bitmap in Pixels						*/
/*  2  Planes;          Number of Planes										*/
/*  2  BitCount;        Bits Per Pixel												*/
/*  4  Compression;     Compression Scheme (0=none)		*/
/*  4  SizeImage;       Size of bitmap in 1s								*/
/*  4  XPelsPerMeter;   Horz. Resolution in Pixels/Meter	*/
/*  4  YPelsPerMeter;   Vert. Resolution in Pixels/Meter		*/
/*  4  ClrUsed;         Number of Colors in Color Table			*/
/*  4  ClrImportant;    Number of Important Colors				*/
/*   																									   */
/**************************************************************/

#ifndef IMAGE_H
#define IMAGE_H



#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <float.h>

#include <im.h>
#include <im_capture.h>
#include <im_format_avi.h>
#include <im_image.h>
#include <im_util.h>

#include <iup.h>
#include <iupcontrols.h>
#include <iupgl.h>


#define ROUND(_) (int)floor( (_) + 0.5 )
#define FLT_MAX         3.402823466e+38F        /* max value */
#define MAX_CORES 256
#define VERM  0
#define VERDE 1
#define AZUL  2



/************************************************************************/
/* Tipos Exportados                                                     */
/************************************************************************/


/* Compiler dependent definitions */
typedef unsigned char BYTE;       
typedef unsigned short int USHORT;  
typedef unsigned short int WORD;  
//     
typedef unsigned long int DWORD;



typedef struct
{
    float r, g, b;
} Color;

typedef struct
{
   Color max;
   Color min;
   Color med;
   Color var1;
   int count;
   int checked;
   int level;
} ColorCube;

struct ColorCubeList
{
   ColorCube *c;
   struct ColorCubeList *next;
   struct ColorCubeList *prev;
};









/**
 *   Imagem com um buffer rgb.
 */



struct Image_imp {

	unsigned int width;		   /* numero de pixels na direcao horizontal da imagem */
	unsigned int height;		   /* numero de pixels na direcao vertical da imagem   */
	//float *buf;       /* vetor de dimensao 3*width*height que armazena consecutivamente as 3 componentes de cor de cada pixel a */
   unsigned char  *buf;           						
   


   /* partir do canto inferior esquerdo da imagem.  A posicao das 3 componentes de cor do pixel (x,y)        */
	/* fica armazenada a partir da posicao (y*width*3) + (x*3)                                                */
};



typedef struct Image_imp Image;

/************************************************************************/
/* Funcoes Exportadas                                                   */
/************************************************************************/

/**
 *	Cria uma nova imagem com as dimensoes especificadas.
 *
 *	@param w Largura da imagem.
 *	@param h Altura da imagem.
 *
 *	@return Handle da imagem criada.
 */
Image  * imgCreate (int w, int h);

/**
 *	Destroi a imagem.
 *
 *	@param image imagem a ser destruida.
 */
void    imgDestroy (Image *image);

/**
 *	Cria uma nova nova copia imagem dada.
 *
 *	@param image imagem a ser copiada.
 *
 *	@return Handle da imagem criada.
 */
Image * imgCopy(Image * image);

/**
 *	Cria uma nova nova copia imagem dada em tons de cinza.
 *
 *	@param image imagem a ser copiada em tons de cinza.
 *
 *	@return Handle da imagem criada.
 */
Image * imgGrey(Image * image);

/**
 *	Obtem a largura (width) de uma imagem.
 *
 *	@param image Handle para uma imagem.
 *	@return  a largura em pixels (width) da imagem.
 */
int imgGetWidth(Image * image);

/**
 *	Obtem a altura (heigth) de uma imagem.
 *
 *	@param image Handle para uma imagem.
 *	@return  a altura em pixels (height) da imagem.
 */
int imgGetHeight(Image * image);

/**
 *	Obtem as dimensoes de uma imagem.
 *
 *	@param image Handle para uma imagem.
 *	@param w [out]Retorna a largura da imagem.
 *	@param h [out]Retorna a altura da imagem.
 */
//float * imgGetRGBData(Image * image);

//void imgSetRGBData(Image * image, float* buffer);

/**
 *	Ajusta o pixel de uma imagem com a cor especificada.
 *
 *	@param image Handle para uma imagem.
 *	@param x Posicao x na imagem.
 *	@param y Posicao y na imagem.
 *	@param color Cor do pixel(valor em float [0,1]).
 */
void imgSetPixel3fv(Image * image, int x, int y, float * color);

/**
 *	Ajusta o pixel de uma imagem com a cor especificada.
 *
 *	@param image Handle para uma imagem.
 *	@param x Posicao x na imagem.
 *	@param y Posicao y na imagem.
 *	@param color Cor do pixel (valor em unsigend char[0,255]).
 */
void imgSetPixel3ubv(Image * image, int x, int y, unsigned char * color);

/**
 *	Obtem o pixel de uma imagem na posicao especificada.
 *
 *	@param image Handle para uma imagem.
 *	@param x Posicao x na imagem.
 *	@param y Posicao y na imagem.
 *	@param color [out] Pixel da posicao especificada(valor em float [0,1]).
 */
void imgGetPixel3fv(Image * image, int x, int y, float *color);

/**
 *	Obtem o pixel de uma imagem na posicao especificada.
 *
 *	@param image Handle para uma imagem.
 *	@param x Posicao x na imagem.
 *	@param y Posicao y na imagem.
 *	@param color [out] Pixel da posicao especificada (valor em unsigend char[0,255]).
 */
void imgGetPixel3ubv(Image * image, int x, int y, unsigned char *color);

/**
 *	Redimensiona a imagem especificada.
 *
 *	@param image Handle para uma imagem.
 *	@param w1 Nova largura da imagem.
 *	@param h1 Nova altura da imagem.
 *  @return imagem criada.
 */
Image * imgResize(Image * img0, int w1, int h1);

/**
 *	Le a imagem a partir do arquivo especificado.
 *
 *	@param filename Nome do arquivo de imagem.
 *
 *	@return imagem criada.
 */
Image * imgReadTGA(char *filename);

/**
 *	Salva a imagem no arquivo especificado em formato TGA.
 *
 *	@param filename Nome do arquivo de imagem.
 *	@param image Handle para uma imagem.
 *
 *	@return retorna 1 caso nao haja erros.
 */
int imgWriteTGA(char *filename, Image * image);


/**
 *	Salva no arquivo especificado a linha h/3 da imagem.
 *
 *	@param image Handle para uma imagem.
 *	@param filename Nome do arquivo no formato texto que contem as cores da linha h/3.
 *
 *	@return retorna 1 caso nao haja erros.
 */
int imgWriteLine(Image * image, char *filename);

/**
 *	Salva no arquivo os histogramas dos 3 canais de cor de [0,255].
 *
 *	@param image Handle para uma imagem.
 *	@param filename Nome do arquivo no formato texto que contera' os 3 histogramas.
 *
 *	@return retorna 1 caso nao haja erros.
 */
int imgWriteHistogram(Image * image, char *filename);

/*** FUNCOES QUE DEVEM SER IMPLEMENTADAS NO TRABALHO 1 ***/

/**
 *	Conta o numero de cores diferentes na imagem
 *
 *	@param image Handle para uma imagem.
 *	@param w Nova largura da imagem.
 *	@param h Nova altura da imagem.
 */
unsigned int imgCountColors(Image * image);

/**
 *	Normaliza as cores da imagem (eliminando o 
 *  efeito da iluminacao.
 *
 *	@param image Handle para uma imagem.
 *  @return Handle para nova imagem normalizada.
 */
Image * imgNormalizeColors(Image * image);

/**
 *	Reduz o numero de cores distintas para 256
 *
 *	@param image Handle para uma imagem.
 *  @return Handle para nova imagem com 256 cores.
 */
Image * imgReduce256Colors(Image * image);


/* img0 =  |img0-img1| */
void imgSub(Image *im0, Image *img1);

/* |image| */
float imgMed(Image *image);

Image * imgAdjust2eN(Image *img0);

/**
 *	Salva a imagem no arquivo especificado em formato BMP.
 *
 *	@param filename Nome do arquivo de imagem.
 *	@param bmp Handle para uma imagem.
 *
 *	@return retorna 1 caso nao haja erros.
 */
imgWriteBMP(char *filename, Image * bmp);

/**
 *	Le a imagem a partir do arquivo especificado.
 *
 *	@param filename Nome do arquivo de imagem.
 *
 *	@return imagem criada.
 */
Image * imgReadBMP (char *filename);

Image * imgGaussFilter(Image * img0);

Image * imgLaplcFilter(Image * img0);

Image * imgEdges(Image * img0);

Image * imgFiltroDeMediana(Image * img0);

Image * imgFiltroDeMedianaH(Image * img0);

Image * imgBinarizacao(Image * img0);

Image * imgCenter(Image * img0, int flagDebug);

//*** Image * imgClear(Image * image, float color[3]);
Image * imgClear(Image * image, unsigned char color[3]);

unsigned char *imgGetRGBData(Image * image);
void imgSetRGBData(Image * image, unsigned char* buffer);

void init_image();



//Image * imgBinarizacao(Image * img0, Image * imgGrey, Image * imgBin)
//int imgBinarizacao(Image * img0);

#endif

