/**
 * jv.h
 * 
 *
 * @authors $Author: $ - Szemberg
 * @version $Id: $
 */


#ifndef _JV_H_
#define _JV_H_

#if defined(__cplusplus)
extern "C" {
#endif

typedef struct {
 float x, y;
 int valid;
 int type;
} jvPoint2D;

typedef struct {
 float v1x, v1y, v2x, v2y;
 int valid;
} jvLine;

typedef int (* PFNRESTRICAOPROC) (int nlines_img, jvLine* lines_img, int nlines_mdl, jvLine* lines_mdl, unsigned char* current_interpretation);

#define MAXPTS 30 /* Nmero mximo de pontos utilizados para encontrar a homografia. */


/*************************************************************************/
//ADDED
jvLine*  getS_lines_mdl ( void );
jvPoint2D* jvCalcHomografia ( float x, float y, float* H );
/*************************************************************************/

/*
Abre a biblioteca de viso 2D que reconhece 
linhas de um modelo presentes em imagens.
Parametros:
 w - largura da imagens em pixels.
 h - altura das imagens em pixels.
 dw - largura das celulas das imagens (usualmente 10).
 dh - altura das celunas das imagens (usualmente 10).
 model_file_name - nome do arquivo que contem o modelo de linhas.
Retorna 0 caso houve algum problema, seno 1.
Obs. dw e dh devem ser divisores de w e h, respectivamente.
*/
int  jvInit(int w, int h, int dw, int dh, char* model_file_name);

/* 
Filtragem LoG em uma imagem monocromtica (img_in), com threshold determinado.
 esperado que as linhas a serem realadas tenham intensidade monocromtica maior que do fundo.
O resultado est colocado no buffer de imagem monocromtica img_out.
O vetor de unsigned char img_out deve ser pr-alocado com a dimenso de w*h.
Na ausencia de um conhecimento melhor sobre a imagem, sugere-se o valor de 20 para o parmetro threshold. 
*/
void jvLoGFilter(unsigned char *img_in, int threshold, unsigned char *img_out);

/* 
Deteco de linhas de uma imagem monocromtica (img), definindo o fator limit_ratio que determina a 
razo mnima entre o maior e menor autovalor que caracterizam as direes principais de cada clula.
Os parmetros dw e dh definem a largura das clulas das imagens (usualmente 10) e a altura das clulas 
das imagens (usualmente 10), respectivamente.
O nmero de linhas detectadas na imagem  fornecido no valor de retorno da funo.
Na ausencia de conhecimento especifico, sugere-se o valor de 20 para o parmetro limit_ratio.
O parmetro colinear_factor  utilizado como fator para teste de colinearidade (sugere-se igual  1.0f). Esse
fator  multiplicado por um definido por FATORCOLINEAR;
*/
int jvEdgeDetect(unsigned char *img, float limit_ratio, jvLine *img_lines, float colinear_factor);

/*
Funo de reconhecimento de correspondecias de linhas baseada numa rvore de interpretao.
Parmetros:
 n_lines_img - nmero de linhas a serem interpretadas;
 lines_img - conjunto de linhas a serem interpretadas;
 eta - fator de distoro perspectiva dada na direo vertical da imagem (sugere-se igual  1.0f). 
Esse fator  multiplicado por um definido por ETA; Vide tese "Acompanhamento de Cenas com Calibrao 
Automtica de Cmeras" equao 4.2.
 ro - tolerncia de paralelismo (sugere-se igual  1.0f). Esse fator  multiplicado por um definido 
por RO; Vide tese "Acompanhamento de Cenas com Calibrao Automtica de Cmeras" equao 4.2.
 restriction_function - restrio adicional;
 response - resultado da interpretao, response[i]=j, significa que a linha i da imagem corresponde a
            linha j do modelo.  255 significa que a linha da imagem no tem correspondencia no modelo.
            Este vetor TEM que ser pr-alocado de tamanho n_lines_img.
*/
void jvInterpret(int n_lines_img, jvLine *lines_img, float eta, float ro, PFNRESTRICAOPROC restriction_function, unsigned char *response);

/*
Funo para ajusta linhas segundo a imagem img.
Parmetros:
 img - imagem;
 n_lines_img - nmero de linhas a serem ajustadas;
 lines_img - conjunto de linhas a serem ajustadas. As linhas so reposicionadas conforme o ajuste.
 tol - Meia distncia, em pixel, dos pontos a serem levados em considerao para o ajuste.
 As linhas que o algoritmo no for capaz de ajustar retornam com o campo valid igual  zero.
*/
void jvAdjust(unsigned char *img, int n_lines_img, jvLine *lines_img, int tol);

/*
Funo para encontrar a tranformao de homografia H[9] (3x3, descrita por linhas) pr-alocada.
a partir de pares de pontos (p1[i], p2[i]), onde n0  o nmero de pares de pontos (limite MAXPTS).
p1 so os pontos do modelo e p2 pontos da imagem capturada e H {p2[i]} = [H]*{p1[i]}
*/
int jvFindTransf(float *H, int n0, jvPoint2D *p1, jvPoint2D *p2);

/*
Funo para desalocar memrias usadas nos mtodos.
*/
void jvClose(void);

#if defined(__cplusplus)
}
#endif

#endif
